from typing import Dict

from starrail.utils.babelfish.locale import Locale
from starrail.utils.loggings import get_logger

logger = get_logger(__file__)


locale_fallback = ['zhs', 'en']


class MultilingualString:

    def __init__(self, **kwargs: str):
        self.mapping: Dict[str, str] = dict()
        for key, value in kwargs.items():
            if key not in Locale.available_lang:
                logger.warn(f'{key} is not a valid language')
            self.mapping[key] = value

    def __call__(self, *args, **kwargs):
        lang = Locale.lang
        if lang not in self.mapping:
            for lang in locale_fallback:
                if lang in self.mapping:
                    break
        return self.mapping[lang].format(*args, **kwargs)



import qfluentwidgets
from PySide6.QtCore import Qt, QUrl
from PySide6.QtGui import QDesktopServices
from PySide6.QtWidgets import QFrame, QHBoxLayout, QLabel, QVBoxLayout, QWidget
from qfluentwidgets import FluentIcon, IconWidget, TextWrap

from starrail.gui.common.stylesheet import StyleSheet

AF = Qt.AlignmentFlag


class LinkCard(QFrame):

    def __init__(self, icon, title, content, url, parent=None):
        super().__init__(parent=parent)
        self.url = QUrl(url)
        self.setFixedSize(198, 220)
        self.iconWidget = IconWidget(icon, self)
        self.titleLabel = QLabel(title, self)
        self.contentLabel = QLabel(TextWrap.wrap(content, 28, False)[0], self)
        self.urlWidget = IconWidget(FluentIcon.LINK, self)

        self.__initWidget()

    def __initWidget(self):
        self.setCursor(Qt.CursorShape.PointingHandCursor)

        self.iconWidget.setFixedSize(54, 54)
        self.urlWidget.setFixedSize(16, 16)

        self.vBoxLayout = QVBoxLayout(self)
        self.vBoxLayout.setSpacing(0)
        self.vBoxLayout.setContentsMargins(24, 24, 0, 13)
        self.vBoxLayout.addWidget(self.iconWidget)
        self.vBoxLayout.addSpacing(16)
        self.vBoxLayout.addWidget(self.titleLabel)
        self.vBoxLayout.addSpacing(8)
        self.vBoxLayout.addWidget(self.contentLabel)
        self.vBoxLayout.setAlignment(AF.AlignLeft | AF.AlignTop)
        self.urlWidget.move(170, 192)

        self.titleLabel.setObjectName('titleLabel')
        self.contentLabel.setObjectName('contentLabel')

    def mouseReleaseEvent(self, e):
        super().mouseReleaseEvent(e)
        QDesktopServices.openUrl(self.url)


class LinkCardView(qfluentwidgets.SingleDirectionScrollArea):
    """ Link card view """

    def __init__(self, parent=None):
        super().__init__(parent, Qt.Orientation.Horizontal)
        self.view = QWidget(self)
        self.hBoxLayout = QHBoxLayout(self.view)

        self.hBoxLayout.setContentsMargins(36, 0, 0, 0)
        self.hBoxLayout.setSpacing(12)
        self.hBoxLayout.setAlignment(AF.AlignLeft)

        self.setWidget(self.view)
        self.setWidgetResizable(True)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)

        self.view.setObjectName('view')
        StyleSheet.LINK_CARD.apply(self)

    def addCard(self, icon, title, content, url):
        """ add link card """
        card = LinkCard(icon, title, content, url, self.view)
        self.hBoxLayout.addWidget(card, 0, AF.AlignLeft)



import qfluentwidgets as qfw
from PySide6 import QtWidgets
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QHBoxLayout, QLabel, QVBoxLayout, QWidget

from starrail.gui.common.stylesheet import StyleSheet

AF = Qt.AlignmentFlag


class ToolBar(QWidget):

    def __init__(self, title, subtitle, parent=None):
        super().__init__(parent=parent)
        self.titleLabel = QLabel(title, self)
        self.subtitleLabel = QLabel(subtitle, self)

        self.vBoxLayout = QVBoxLayout(self)

        self.__initWidget()

    def __initWidget(self):
        self.setFixedHeight(98)
        self.vBoxLayout.setSpacing(0)
        self.vBoxLayout.setContentsMargins(36, 22, 12, 12)
        self.vBoxLayout.addWidget(self.titleLabel)
        self.vBoxLayout.addSpacing(4)
        self.vBoxLayout.addWidget(self.subtitleLabel)
        self.vBoxLayout.setAlignment(AF.AlignTop)

        self.titleLabel.setObjectName('titleLabel')
        self.subtitleLabel.setObjectName('subtitleLabel')


class CardWidget(QWidget):

    def __init__(self, title, parent=None):
        super().__init__(parent=parent)

        self.titleLabel = QLabel(title, self)
        self.card = QtWidgets.QFrame(self)
        self.vBoxLayout = QVBoxLayout(self)
        self.cardLayout = QVBoxLayout(self.card)
        self.topLayout = QHBoxLayout()

        self.__initLayout()
        self.__initWidget()

    def __initWidget(self):
        self.titleLabel.setObjectName('titleLabel')
        self.card.setObjectName('card')

    def __initLayout(self):
        QVBL_SC = QVBoxLayout.SizeConstraint
        QHBL_SC = QHBoxLayout.SizeConstraint
        self.vBoxLayout.setSizeConstraint(QVBL_SC.SetMinimumSize)
        self.cardLayout.setSizeConstraint(QVBL_SC.SetMinimumSize)
        self.topLayout.setSizeConstraint(QHBL_SC.SetMinimumSize)

        self.vBoxLayout.setSpacing(12)
        self.vBoxLayout.setContentsMargins(0, 0, 0, 0)
        self.topLayout.setContentsMargins(12, 12, 12, 12)
        self.cardLayout.setContentsMargins(0, 0, 0, 0)

        self.vBoxLayout.addWidget(self.titleLabel, 0, AF.AlignTop)
        self.vBoxLayout.addWidget(self.card, 0, AF.AlignTop)
        self.vBoxLayout.setAlignment(AF.AlignTop)

        self.cardLayout.setSpacing(0)
        self.cardLayout.setAlignment(AF.AlignTop)
        self.cardLayout.addLayout(self.topLayout, 0)

    def addWidget(self, widget: QWidget):
        widget.setParent(self.card)
        self.topLayout.addWidget(widget)
        widget.show()

    def addStretch(self, stretch: int):
        self.topLayout.addStretch(stretch)

    def addSpacing(self, spacing: int):
        self.topLayout.addSpacing(spacing)


class BaseInterface(qfw.ScrollArea):

    def __init__(self, title: str, subtitle: str, parent: QWidget = None):
        super().__init__(parent=parent)
        self.view = QWidget(self)
        self.toolBar = ToolBar(title, subtitle, self)
        self.vBoxLayout = QVBoxLayout(self.view)

        SBP = Qt.ScrollBarPolicy
        self.setHorizontalScrollBarPolicy(SBP.ScrollBarAlwaysOff)
        self.setViewportMargins(0, self.toolBar.height(), 0, 0)
        self.setWidget(self.view)
        self.setWidgetResizable(True)

        self.vBoxLayout.setSpacing(30)
        self.vBoxLayout.setAlignment(AF.AlignTop)
        self.vBoxLayout.setContentsMargins(36, 20, 36, 36)

        self.view.setObjectName('view')
        StyleSheet.BASE_INTERFACE.apply(self)

    def addCard(self, title):
        card = CardWidget(title=title, parent=self)
        self.vBoxLayout.addWidget(card, 0, AF.AlignTop)
        return card

    def resizeEvent(self, e):
        super().resizeEvent(e)
        self.toolBar.resize(self.width(), self.toolBar.height())



import logging
import os

from starrail import package_path


def setup_logging(log_level):
    format = (
        '%(asctime)s %(name)s Line %(lineno)d - '
        '[%(levelname)s] %(message)s'
    )
    formatter = logging.Formatter(format)

    stream = logging.StreamHandler()
    stream.setFormatter(formatter)
    logging.getLogger().addHandler(stream)
    logging.getLogger().setLevel(log_level)


def get_logger(name):
    if name and os.path.isfile(name):
        name = os.path.relpath(name, package_path)
    return logging.getLogger(name)



import json
import traceback

import requests

from starrail.utils import loggings

logger = loggings.get_logger(__file__)


def fetch_json(url, timeout=None):  # no typing annotations to pass mypy check
    """
    Fetches JSON content from a URL and returns it as a dictionary.

    Args:
        url: A string representing the URL to fetch the JSON content from.

    Returns:
        A dictionary representing the JSON content fetched from the URL.
    """

    try:
        r = requests.get(url, timeout=timeout)
        if 200 <= r.status_code < 300:
            content = r.content.decode('utf-8', errors='ignore')
            payload = json.loads(content)
            logger.debug(json.dumps(payload, ensure_ascii=False, indent=2))
            return payload, r.status_code
    except Exception:
        traceback.print_exc()
    return None, -1


def fetch_text(url):
    try:
        r = requests.get(url)
        if 200 <= r.status_code < 300:
            content = r.content.decode('utf-8', errors='ignore')
            return content, r.status_code
    except Exception:
        traceback.print_exc()
    return None, -1



import sys

from starrail.config import configuration as cfg
from starrail.entry.setup import setup
from starrail.utils import loggings

logger = loggings.get_logger(__file__)

try:
    from PySide6.QtCore import Qt
    from PySide6.QtWidgets import QApplication

    from starrail.gui.application import StarRailToolkit
    from starrail.gui.common.config import qcfg
except ImportError as e:
    logger.critical(
        'Graphic mode is defaultly not enabled. If you want to run the GUI, '
        'please download the executables or launch from the source code. '
        'See: https://github.com/LittleNyima/honkai-starrail-toolkit for '
        'further information.',
    )
    raise e


def gui_entry():
    setup(log_level=cfg.log_level, locale=qcfg.locale.value.value.name())

    AA = Qt.ApplicationAttribute
    app = QApplication(sys.argv)
    app.setAttribute(AA.AA_DontCreateNativeWidgetSiblings)

    window = StarRailToolkit()
    window.show()
    window.afterShow()

    sys.exit(app.exec())


if __name__ == '__main__':
    gui_entry()



from typing import Dict, List, Optional
from urllib.parse import parse_qs, urlencode, urlparse, urlunparse

from starrail.utils import loggings

logger = loggings.get_logger(__file__)
QueryDictType = Dict[str, List[str]]


def get_clean_query_dict(query_dict: QueryDictType) -> QueryDictType:
    """
    Given a dictionary of query parameters, remove any backslashes in the
    keys and values and return a new dictionary with the cleaned keys and
    values.

    Args:
        query_dict (QueryDictType): A dictionary containing query parameters

    Returns:
        QueryDictType: A new dictionary with the cleaned keys and values
    """

    clean_query = dict()
    for k, v in query_dict.items():
        clean_k = k.replace('\\', '')
        clean_v = [item.replace('\\', '') for item in v]
        clean_query[clean_k] = clean_v
    return clean_query


def pop_if_exist(query_dict: QueryDictType, key: str) -> Optional[List[str]]:
    """
    Given a dictionary of query parameters and a key, remove the key from the
    dictionary if it exists and return its corresponding value. If the key
    does not exist, return None.

    Args:
        query_dict (QueryDictType): A dictionary containing query parameters
        key (str): The key to remove from the dictionary

    Returns:
        Optional[str]: The corresponding value of the key that was removed,
            or None if the key did not exist
    """

    if key in query_dict:
        return query_dict.pop(key)
    return None


def get_url_template(url: str) -> str:
    """
    Given a URL string, remove specific query parameters
    ('end_id', 'gacha_type', 'page', 'size'),
    remove any backslashes from the remaining query parameters,
    and return a new URL string with the cleaned query parameters.

    Args:
        url (str): A URL string containing query parameters

    Returns:
        str: A new URL string with the cleaned query parameters
    """

    parse = urlparse(url)
    query_dict = parse_qs(parse.query)
    query_dict = get_clean_query_dict(query_dict)
    pop_if_exist(query_dict, 'begin_id')
    pop_if_exist(query_dict, 'end_id')
    pop_if_exist(query_dict, 'gacha_type')
    pop_if_exist(query_dict, 'page')
    pop_if_exist(query_dict, 'size')
    new_query = urlencode(query_dict, doseq=True)
    return urlunparse((
        parse.scheme, parse.netloc, parse.path, parse.params,
        new_query, parse.fragment,
    ))


def get_api_url(
    template: str, end_id: str, gacha_type: str, page: str,
    size: str,
) -> str:
    """
    Constructs and returns an API URL based on the given template and
    parameters.

    Args:
        template (str): The base URL template to be used as a starting point.
        end_id (str): The ending ID for the desired data range.
        gacha_type (str): The type of gacha data to retrieve.
        page (str): The page number of results to return.
        size (str): The number of results per page to return.

    Returns:
        str: A string representing the complete API URL with all necessary
            parameters included.
    """

    parse = urlparse(template)
    query_dict = parse_qs(parse.query)
    query_dict.update({
        'end_id': [end_id],
        'gacha_type': [gacha_type],
        'page': [page],
        'size': [size],
    })
    new_query = urlencode(query_dict, doseq=True)
    return urlunparse((
        parse.scheme, parse.netloc, parse.path, parse.params,
        new_query, parse.fragment,
    ))



from enum import Enum


class GachaType(Enum):
    """
    Enumeration class representing the various types of gacha types.

    Attributes:
        STELLAR (int): Value representing the Stellar Event Warp.
        DEPARTURE (int): Value representing the Departure Event Warp.
        CHARACTER (int): Value representing the Character Event Warp.
        LIGHT_CONE (int): Value representing the Light Cone Event Warp.
    """

    STELLAR = 1
    DEPARTURE = 2
    CHARACTER = 11
    LIGHT_CONE = 12



import qfluentwidgets
from PySide6.QtCore import QEasingCurve, Qt, Signal
from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QApplication, QFrame, QHBoxLayout, QWidget
from qfluentwidgets import NavigationInterface, NavigationItemPosition
from qframelesswindow import FramelessWindow

from starrail.gui.common.config import qcfg
from starrail.gui.common.icon import Icon
from starrail.gui.common.stylesheet import StyleSheet
from starrail.gui.common.utils import checkUpdate
from starrail.gui.interfaces.gacha_sync import GachaSyncInterface
from starrail.gui.interfaces.home import HomeInterface
from starrail.gui.interfaces.setting import SettingInterface
from starrail.gui.interfaces.unlock_fps import UnlockFpsInterface
from starrail.gui.interfaces.users import UsersInterface
from starrail.gui.widgets.title_bar import CustomTitleBar
from starrail.utils import babelfish


class StackedWidget(QFrame):

    currentWidgetChanged = Signal(QWidget)

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.hBoxLayout = QHBoxLayout(self)
        self.view = qfluentwidgets.PopUpAniStackedWidget(self)

        self.hBoxLayout.setContentsMargins(0, 0, 0, 0)
        self.hBoxLayout.addWidget(self.view)

        self.view.currentChanged.connect(
            lambda i: self.currentWidgetChanged.emit(self.view.widget(i)),
        )

    def addWidget(self, widget):
        """ add widget to view """
        self.view.addWidget(widget)

    def setCurrentWidget(self, widget, popOut=False):
        widget.verticalScrollBar().setValue(0)
        if not popOut:
            self.view.setCurrentWidget(widget, duration=300)
        else:
            self.view.setCurrentWidget(
                widget, True, False, 200, QEasingCurve.InQuad,
            )

    def setCurrentIndex(self, index, popOut=False):
        self.setCurrentWidget(self.view.widget(index), popOut)


class StarRailToolkit(FramelessWindow):

    def __init__(self):
        super().__init__()
        self.setTitleBar(CustomTitleBar(self))
        self.hBoxLayout = QHBoxLayout(self)
        self.widgetLayout = QHBoxLayout()

        self.stackWidget = StackedWidget(self)
        self.navigationInterface = NavigationInterface(self, True, True)

        self.homeInterface = HomeInterface(self)
        self.gachaSyncInterface = GachaSyncInterface(
            babelfish.ui_gacha_sync(),
            babelfish.ui_gacha_sync_desc(),
            self,
        )
        self.unlockFpsInterface = UnlockFpsInterface(
            babelfish.ui_unlock_fps(),
            babelfish.ui_unlock_fps_desc(),
            self,
        )
        self.usersInterface = UsersInterface(
            babelfish.ui_users(),
            babelfish.ui_users_desc(),
            self,
        )
        self.settingInterface = SettingInterface(self)

        self.initLayout()
        self.initNavigation()
        self.initWindow()

    def initLayout(self):
        self.hBoxLayout.setSpacing(0)
        self.hBoxLayout.setContentsMargins(0, 0, 0, 0)
        self.hBoxLayout.addWidget(self.navigationInterface)
        self.hBoxLayout.addLayout(self.widgetLayout)
        self.hBoxLayout.setStretchFactor(self.widgetLayout, 1)

        self.widgetLayout.addWidget(self.stackWidget)
        self.widgetLayout.setContentsMargins(0, 48, 0, 0)

        self.navigationInterface.displayModeChanged.connect(
            self.titleBar.raise_,
        )
        self.titleBar.raise_()

    def initNavigation(self):
        self.addSubInterface(
            self.homeInterface,
            'homeInterface',
            qfluentwidgets.FluentIcon.HOME,
            babelfish.ui_welcome(),
            NavigationItemPosition.TOP,
        )

        self.navigationInterface.addSeparator()

        self.addSubInterface(
            self.gachaSyncInterface,
            'gachaSyncInterface',
            qfluentwidgets.FluentIcon.SYNC,
            babelfish.ui_gacha_sync(),
            NavigationItemPosition.TOP,
        )

        self.addSubInterface(
            self.unlockFpsInterface,
            'unlockFpsInterface',
            Icon.UNLOCK,
            'unlock fps',
            NavigationItemPosition.TOP,
        )

        self.addSubInterface(
            self.usersInterface,
            'usersInterface',
            Icon.USER,
            babelfish.ui_users(),
            NavigationItemPosition.BOTTOM,
        )

        self.addSubInterface(
            self.settingInterface,
            'settingInterface',
            qfluentwidgets.FluentIcon.SETTING,
            babelfish.ui_settings(),
            NavigationItemPosition.BOTTOM,
        )

        self.navigationInterface.setDefaultRouteKey(
            self.homeInterface.objectName(),
        )
        self.stackWidget.currentWidgetChanged.connect(
            lambda w: self.navigationInterface.setCurrentItem(w.objectName()),
        )
        self.navigationInterface.setCurrentItem(
            self.homeInterface.objectName(),
        )
        self.stackWidget.setCurrentIndex(0)

    def addSubInterface(
        self,
        interface: QWidget,
        objectName: str,
        icon,
        text: str,
        position=NavigationItemPosition.SCROLL,
    ):
        """ add sub interface """
        interface.setObjectName(objectName)
        self.stackWidget.addWidget(interface)
        self.navigationInterface.addItem(
            routeKey=objectName,
            icon=icon,
            text=text,
            onClick=lambda t: self.switchTo(interface, t),
            position=position,
            tooltip=text,
        )

    def switchTo(self, widget, triggerByUser=True):
        self.stackWidget.setCurrentWidget(widget, not triggerByUser)

    def initWindow(self):
        self.resize(960, 640)
        self.setMinimumSize(960, 600)
        self.setWindowIcon(QIcon('resources/images/logo.jpg'))
        self.setWindowTitle(babelfish.ui_title())
        self.titleBar.setAttribute(Qt.WidgetAttribute.WA_StyledBackground)

        desktop = QApplication.screens()[0].availableGeometry()
        w, h = desktop.width(), desktop.height()
        self.move(w // 2 - self.width() // 2, h // 2 - self.height() // 2)

        StyleSheet.STAR_RAIL_TOOLKIT.apply(self)

    def resizeEvent(self, _):
        self.titleBar.move(46, 0)
        self.titleBar.resize(self.width() - 46, self.titleBar.height())

    def afterShow(self):
        if qcfg.get(qcfg.check_update):
            checkUpdate(self)



import json
import time

import pandas as pd

import starrail
import starrail.utils.babelfish as babelfish
from starrail.gacha.database import DatabaseFactory
from starrail.gacha.parse import GachaDataManager
from starrail.gacha.type import GachaType
from starrail.utils import loggings

logger = loggings.get_logger(__file__)


def export_as_sql(manager: GachaDataManager, output_path: str) -> None:
    logger.info(f'Exporting data to cache {output_path}')
    with DatabaseFactory.get_database(output_path) as db:
        for gacha_type in GachaType:
            should_insert = []
            for entry in manager.gacha[gacha_type.value]:
                if 'existing' in entry and not entry['existing']:
                    should_insert.append(entry)
            logger.info(
                f'Exporting {gacha_type.name}, '
                f'totally {len(should_insert)} new items',
            )
            rev_insert = reversed(should_insert)
            for entry in rev_insert:
                db.add_entry(gacha_type.name, entry)


def export_as_json(manager: GachaDataManager, output_path: str) -> None:
    data_dict = dict()
    for gacha_type in GachaType:
        data_dict[gacha_type.name] = manager.gacha[gacha_type.value].tolist()
    with open(output_path, 'w', encoding='utf-8') as fout:
        json.dump(data_dict, fout, indent=2, ensure_ascii=False)


def export_as_xlsx(manager: GachaDataManager, output_path: str) -> None:
    with pd.ExcelWriter(output_path) as writer:
        for gacha_type in GachaType:
            df = pd.json_normalize(manager.gacha[gacha_type.value].tolist())
            df.to_excel(writer, sheet_name=gacha_type.name)


def export_as_csv(manager: GachaDataManager, output_path: str) -> None:
    data_list = []
    for gacha_type in GachaType:
        data_list.extend(manager.gacha[gacha_type.value].tolist())
    df = pd.json_normalize(data_list)
    df.to_csv(output_path, encoding='utf-8')


def export_as_md(manager: GachaDataManager, output_path: str) -> None:
    md = f'# {babelfish.gacha_report()}\n\n'
    for gacha_type in GachaType:
        md += f'## {babelfish.translate(gacha_type.name)}\n\n'
        md += babelfish.markdown_thead()
        stats = manager.gacha[gacha_type.value].stats
        for item in stats:
            rtype = item['rank_type']
            count = item['count']
            basic = item['basic_prob']
            compr = item['compr_prob']
            since_last = item['since_last']
            md += f'|{rtype}|{count}|{basic}|{compr}|{since_last}|\n'
        md += '\n'
        if stats[0]['attempts']:
            attempt_string = ' '.join(stats[0]['attempts'])
            average = stats[0]['average']
            md += f'{babelfish.history_of_5_stars()}: '
            md += f'**{attempt_string}**\n\n'
            md += f'{babelfish.average_gacha_per_5_star()}: **{average}**\n\n'
    with open(output_path, 'w', encoding='utf-8') as fout:
        fout.write(md)


def export_as_html(manager: GachaDataManager, output_path: str) -> None:
    title = f'{babelfish.gacha_title(manager.uid)}'
    content = f'<h1>{babelfish.gacha_report()}</h1>\n'
    for gacha_type in GachaType:
        content += f'<h2>{babelfish.translate(gacha_type.name)}</h2>\n'
        thead = babelfish.html_thead()
        table = f'<table><thead><tr>{thead}</tr></thead><tbody>\n'
        stats = manager.gacha[gacha_type.value].stats
        for item in stats:
            rtype = item['rank_type']
            count = item['count']
            basic = item['basic_prob']
            compr = item['compr_prob']
            since_last = item['since_last']
            table += (
                f'<tr><td>{rtype}</td><td>{count}</td><td>{basic}</td>'
                f'<td>{compr}</td><td>{since_last}</td></tr>\n'
            )
        table += '</tbody></table>\n'
        content += table
        if stats[0]['attempts']:
            attempt_string = ' '.join(stats[0]['attempts'])
            average = stats[0]['average']
            content += f'<p>{babelfish.history_of_5_stars()}: '
            content += f'<b>{attempt_string}</b></p>\n'
            content += f'<p>{babelfish.average_gacha_per_5_star()}: '
            content += f'<b>{average}</b></p>\n'
    style = (
        '<style type="text/css">'
        r'html {font-family: sans-serif;} '
        r'body {margin: 10px;} '
        r'table {border-collapse: collapse; border-spacing: 0;'
        r'       empty-cells: show; border: 1px solid #cbcbcb;} '
        r'td, th {padding: 0; border-left: 1px solid #cbcbcb;'
        r'        border-width: 0 0 0 1px; font-size: inherit;'
        r'        margin: 0; overflow: visible; padding: .5em 1em;} '
        r'thead {background-color: #e0e0e0; color: #000;'
        r'       text-align: left; vertical-align: bottom; }'
        r'tr:nth-child(odd) td {background-color: transparent;} '
        r'tr:nth-child(even) td {background-color: #f2f2f2;} '
        '</style>'
    )
    html = (
        f'<html lang="zh"><head><title>{title}</title><meta charset="UTF-8">'
        f'{style}</head><body>{content}</body></html>'
    )
    with open(output_path, 'w', encoding='utf-8') as fout:
        fout.write(html)


def export_as_srgf(manager: GachaDataManager, output_path: str) -> None:
    gacha_list = []
    for gacha_type in GachaType:
        gacha_list.extend(manager.gacha[gacha_type.value].tolist())
    gacha_list.sort(key=lambda x: x['id'])
    uid = gacha_list[0]['uid'] if gacha_list else manager.uid
    lang = gacha_list[0]['lang'] if gacha_list else 'zh-cn'
    timezone = time.localtime().tm_gmtoff // 3600
    timestamp = int(time.time())
    data = dict(
        info=dict(
            uid=uid,
            lang=lang,
            region_time_zone=timezone,
            export_timestamp=timestamp,
            export_app='StarRailToolkit',
            export_app_version=starrail.__version__,
            srgf_version='v1.0',
        ),
        list=[
            dict(
                gacha_id=item['gacha_id'],
                gacha_type=item['gacha_type'],
                item_id=item['item_id'],
                count=item['count'],
                time=item['time'],
                name=item['name'],
                item_type=item['item_type'],
                rank_type=item['rank_type'],
                id=item['id'],
            ) for item in gacha_list
        ],
    )
    with open(output_path, 'w', encoding='utf-8') as fout:
        json.dump(data, fout, indent=2, ensure_ascii=False)



import easydict

from starrail.gacha.fetch import fetch_json

user = 'LittleNyima'
repo = 'honkai-starrail-toolkit'
branch = 'master'
cdn_templ = dict(
    gitee=fr'https://gitee.com/{user}/{repo}-mirror/raw/{branch}/',
    github=fr'https://raw.githubusercontent.com/{user}/{repo}/{branch}/',
    jsdelivr=fr'https://cdn.jsdelivr.net/gh/{user}/{repo}@{branch}/',
)
# jsdelivr is deprecated due to untimely synchronization


"""
Distribution JSON Format:
{
    'version': 'the-latest-version',
    'dist': {
        'name-1': 'url-1',
        'name-2': 'url-2',
    },
    'changelog': 'desc'
}
"""


def get_distribution(cdn_type):
    distribution = fr'{cdn_templ[cdn_type]}releases/dist.json'
    return fetch_json(distribution, timeout=5)


def check_update():
    for cdn_type in ['gitee', 'github']:
        payload, _ = get_distribution(cdn_type)
        if payload is not None:
            return easydict.EasyDict(payload)
    return None



import configparser
import os
import platform
import re
from urllib.parse import parse_qsl, urlparse

from starrail.utils import loggings
from starrail.utils.misc import lazy_import

logger = loggings.get_logger(__file__)


reg_key_prefix = 'SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\'
reg_key_cn = reg_key_prefix + '崩坏：星穹铁道'
# reg_key_os = reg_key_prefix + 'Star Rail'
api_pattern = re.compile(r'https://.+/api/getGachaLog.+game_biz=hkrpg.+')


def detect_game_install_path():
    logger.info('Detecting game install path')
    winreg = lazy_import('winreg')
    with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, reg_key_cn) as rk:
        install_path = winreg.QueryValueEx(rk, 'InstallPath')[0]
    config_path = os.path.join(install_path, 'config.ini')
    parser = configparser.ConfigParser()
    parser.read(config_path)
    game_install_path = parser.get('launcher', 'game_install_path')
    if not os.path.exists(game_install_path):
        error_msg = (
            f'Game install path {game_install_path} is not existing. '
            'Please check whether the game is installed correctly.'
        )
        logger.error(error_msg)
    return game_install_path


def get_cache_path(game_install_path):
    logger.info('Getting gacha query cache path')
    cache_path = os.path.join(
        game_install_path, 'StarRail_Data', 'webCaches',
        'Cache', 'Cache_Data', 'data_2',
    )
    if not os.path.exists(cache_path):
        error_msg = (
            f'Cache path {cache_path} is not existing. Please visit '
            'the gacha querying page before exporting gacha data.'
        )
        logger.error(error_msg)
    return cache_path


def get_url_from_text(text):
    if not text:
        return None
    urls = re.findall(api_pattern, text)
    if not urls:
        return None
    return urls[-1]


def safe_int(value, default_value=0):
    try:
        return int(value)
    except Exception:
        return default_value


def get_timestamp_from_url(url):
    parse = urlparse(url)
    query_dict = dict(parse_qsl(parse.query))
    timestamp = query_dict.get('timestamp', 0)
    return safe_int(timestamp)


def get_latest_url(url_list):
    timestamp_list = list(map(get_timestamp_from_url, url_list))
    max_timestamp = max(timestamp_list)
    max_index = [
        idx for idx, timestamp in enumerate(timestamp_list)
        if timestamp == max_timestamp
    ]
    return url_list[max_index[-1]]


# Modified from: https://github.com/sunfkny/genshin-gacha-export
def get_api_from_cache(cache_path):
    logger.info('Getting api URL from cache')
    with open(cache_path, 'rb') as f_cache:
        cache = f_cache.read()
    parts = cache.split(b'1/0/')
    parts = [part.split(b'\x00')[0].decode(errors='ignore') for part in parts]
    parts = map(get_url_from_text, parts)
    parts = list(filter(bool, parts))
    if not parts:
        error_msg = (
            'API URL is not found in cache. Please visit the gacha querying '
            'page before exporting gacha data.'
        )
        logger.error(error_msg)
        return ''
    return get_latest_url(parts)


def detect_api_url():
    if platform.system() == 'Windows':
        logger.info('Trying to auto-detecting API URL')
        game_install_path = detect_game_install_path()
        cache_path = get_cache_path(game_install_path)
        return get_api_from_cache(cache_path)
    else:
        logger.error(
            'Auto-detect API URL is only supported on Windows '
            'platform, aborting.',
        )



from enum import Enum

from qfluentwidgets import StyleSheetBase, Theme, qconfig


class StyleSheet(StyleSheetBase, Enum):
    """ Style sheet  """

    BASE_INTERFACE = 'base_interface'
    CHECK_UPDATE_DIALOG = 'check_update_dialog'
    HOME_INTERFACE = 'home_interface'
    LINK_CARD = 'link_card'
    RESULT_TABLE_WIDGET = 'result_table_widget'
    SETTING_INTERFACE = 'setting_interface'
    STAR_RAIL_TOOLKIT = 'star_rail_toolkit'

    def path(self, theme=Theme.AUTO):
        theme = qconfig.theme if theme == Theme.AUTO else theme
        return f'resources/qss/{theme.value.lower()}/{self.value}.qss'



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel, QVBoxLayout, QWidget
from qfluentwidgets import FluentIcon, ScrollArea

from starrail.gui.common.stylesheet import StyleSheet
from starrail.gui.widgets.link_card import LinkCardView
from starrail.utils import babelfish


class BannerWidget(QWidget):
    """ Banner widget """

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setFixedHeight(336)
        self.vBoxLayout = QVBoxLayout(self)
        self.galleryLabel = QLabel(babelfish.ui_welcome(), self)
        self.linkCardView = LinkCardView(self)

        self.galleryLabel.setObjectName('galleryLabel')

        self.vBoxLayout.setSpacing(0)
        self.vBoxLayout.setContentsMargins(0, 20, 0, 0)
        self.vBoxLayout.addWidget(self.galleryLabel)
        self.vBoxLayout.addWidget(self.linkCardView, 1, Qt.AlignBottom)
        self.vBoxLayout.setAlignment(Qt.AlignLeft | Qt.AlignTop)

        self.linkCardView.addCard(
            FluentIcon.BOOK_SHELF,
            babelfish.ui_get_started(),
            babelfish.ui_get_started_desc(),
            babelfish.constants.DOCS_URL,
        )

        self.linkCardView.addCard(
            FluentIcon.HELP,
            babelfish.ui_troubleshooting(),
            babelfish.ui_troubleshooting_desc(),
            babelfish.constants.TROUBLESHOOTING_URL,
        )

        self.linkCardView.addCard(
            FluentIcon.GITHUB,
            babelfish.ui_github_repo(),
            babelfish.ui_github_repo_desc(),
            babelfish.constants.REPO_URL,
        )

        self.linkCardView.addCard(
            FluentIcon.FEEDBACK,
            babelfish.ui_send_feedback(),
            babelfish.ui_send_feedback_desc(),
            babelfish.constants.ISSUE_URL,
        )


class HomeInterface(ScrollArea):
    """ Home interface """

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.banner = BannerWidget(self)
        self.view = QWidget(self)
        self.vBoxLayout = QVBoxLayout(self.view)

        self.__initWidget()

    def __initWidget(self):
        self.view.setObjectName('view')
        StyleSheet.HOME_INTERFACE.apply(self)

        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setWidget(self.view)
        self.setWidgetResizable(True)

        self.vBoxLayout.setContentsMargins(0, 0, 0, 36)
        self.vBoxLayout.setSpacing(40)
        self.vBoxLayout.addWidget(self.banner)
        self.vBoxLayout.setAlignment(Qt.AlignTop)



import json
import os

from easydict import EasyDict

userroot = os.path.abspath(os.path.expanduser('~'))


class Configuration(EasyDict):

    def __init__(self, d=None, **kwargs):
        self.no_flush = True
        self.skip_keys = set()
        self.user_keys = set(kwargs.keys())
        super().__init__(d, **kwargs)
        self.no_flush = False

    def set_skip_keys(self, *args: str):
        self.skip_keys |= set(args)

    def __setattr__(self, name, value):
        super().__setattr__(name, value)
        if (
            not self.no_flush
            and name in self.user_keys
            and name not in self.skip_keys
        ):
            self.flush()

    def flush(self):
        if not self.no_flush:
            cfg = {
                k: self[k]
                for k in self.user_keys if k not in self.skip_keys
            }
            with open(self.config_path, 'w', encoding='utf-8') as fcfg:
                json.dump(cfg, fcfg, indent=2)


configuration = Configuration(
    cache_dir=os.path.join(userroot, '.starrail'),
    db_dir=os.path.join(userroot, '.starrail', 'database'),
    config_path=os.path.join(userroot, '.starrail', 'config.json'),
    account_record_path=os.path.join(userroot, '.starrail', 'accounts.json'),
    check_update=True,
    locale='zhs',
    log_level='DEBUG',
)
configuration.set_skip_keys(
    'skip_keys', 'no_flush',
    'cache_dir', 'config_path', 'db_dir', 'account_record_path',
)


def init_config():

    os.makedirs(configuration.cache_dir, exist_ok=True)
    os.makedirs(configuration.db_dir, exist_ok=True)

    if os.path.isfile(configuration.config_path):
        with open(configuration.config_path, encoding='utf-8') as fcfg:
            custom_config = json.load(fcfg)

        configuration.update(custom_config)
    else:
        configuration.flush()



import os

from .version import __version__, digital_version

package_path = os.path.dirname(__file__)


def _welcome():
    print('+--------------------------------------+')
    print('|      Honkai: Star Rail Toolkit       |')
    print('|      ^^^^^^^^^^^^^^^^^^^^^^^^^       |')
    print('|         author: LittleNyima          |')
    print('+--------------------------------------+')
    print(
        'NOTICE: This software is licensed under the GNU General Public\n'
        'License v3.0 and SHALL NOT be used for commercial purpose. All\n'
        'rights reserved to LittleNyima. Please contact me if the project\n'
        'has any potential infringement risks.',
    )


_welcome()


__all__ = ['__version__', 'digital_version', 'package_path']



import platform
import sys

import starrail
from starrail.config import init_config
from starrail.utils.babelfish.locale import setup_locale
from starrail.utils.loggings import get_logger, setup_logging

logger = get_logger(__file__)


def display_platform_info():
    logger.info(f'PLATFORM:     {platform.platform()}')
    logger.info(f'VERSION:      {platform.version()}')
    logger.info(f'ARCHITECTURE: {platform.architecture()}')
    logger.info(f'MACHINE:      {platform.machine()}')
    logger.info(f'NODE:         {platform.node()}')
    logger.info(f'PROCESSOR:    {platform.processor()}')
    logger.info(f'SYSTEM:       {platform.system()}')
    logger.info(f'RELEASE:      {platform.release()}')


def display_python_info():
    logger.info(f'PYTHON_BRANCH:         {platform.python_branch()}')
    logger.info(f'PYTHON_BUILD:          {platform.python_build()}')
    logger.info(f'PYTHON_COMPILER:       {platform.python_compiler()}')
    logger.info(f'PYTHON_IMPLEMENTATION: {platform.python_implementation()}')
    logger.info(f'PYTHON_REVISION:       {platform.python_revision()}')
    logger.info(f'PYTHON_VERSION:        {platform.python_version()}')


def display_package_info():
    logger.info(f'EXECUTABLE: {sys.executable}')
    logger.info(f'PACKAGE_VERSION: {starrail.__version__}')


def setup(log_level, locale):
    setup_logging(log_level)
    setup_locale(locale)
    init_config()
    display_platform_info()
    display_python_info()
    display_package_info()



import json
import os
import time

import starrail.gacha.fileio as fileio
from starrail.gacha.autodet import detect_api_url
from starrail.gacha.fetch import fetch_json
from starrail.gacha.parse import GachaDataManager
from starrail.gacha.type import GachaType
from starrail.gacha.url import get_api_url, get_url_template
from starrail.utils import loggings
from starrail.utils.accounts import account_record

logger = loggings.get_logger(__file__)


def integers():
    r = 1
    while True:
        yield r
        r += 1


def check_response(payload, code):
    # returns: is_valid, should_stop, massage
    resp = json.dumps(payload, ensure_ascii=False)
    if payload is None:  # check payload
        return False, True, f'Payload is None, response: {resp}'
    elif not 200 <= code < 300:  # check request
        return False, True, f'Request fail, code: {code}, response: {resp}'
    elif 'data' not in payload:  # check data
        return False, True, f'data is not in payload, response: {resp}'
    elif not payload['data']:  # check data empty or null
        return False, True, f'data is empty or None, response: {resp}'
    elif 'list' not in payload['data']:  # check field
        return False, True, f'data.list is missing, response: {resp}'
    elif not payload['data']['list']:  # check data list
        return True, True, 'valid but reaching the end of list'
    return True, False, 'ok'


def export_gacha_type(
    api_template: str,
    gacha_type: GachaType,
    request_interval: float,
):
    r = []
    end_id = '0'
    for page in integers():
        logger.info(f'Downloading page {page} of type {gacha_type.name}')
        api_url = get_api_url(
            api_template, end_id, str(gacha_type.value),
            str(page), '5',
        )
        logger.debug(f'Requesting {api_url}')
        response, code = fetch_json(api_url)
        time.sleep(request_interval)
        _, should_stop, msg = check_response(response, code)
        logger.info(f'check_response: {msg}')
        if should_stop:
            break
        data_list = response['data']['list']
        r.extend(data_list)
        end_id = data_list[-1]['id']
    return r


def deduce_uid(record_cache):
    for gacha_type in GachaType:
        records = record_cache[gacha_type.value]
        if records:
            return records[0]['uid']
    return ''


def export_gacha_from_api(api_url, export, request_interval):
    if not api_url:
        api_url = detect_api_url()
    response, code = fetch_json(api_url)
    valid, _, msg = check_response(response, code)
    logger.info(f'check_response (api): {msg}')
    if not valid:
        logger.fatal('Error while checking response from api URL, exitting')
        raise ValueError('Invalid or expired api, please check your input')

    api_template = get_url_template(api_url)
    record_cache = dict()

    for gacha_type in GachaType:
        records = export_gacha_type(
            api_template,
            gacha_type,
            request_interval,
        )
        record_cache[gacha_type.value] = records
        logger.info(f'Finish downloading records of {gacha_type.name}')

    uid = deduce_uid(record_cache)
    if not uid:
        logger.fatal(
            'Cannot deduce uid from records, there may be no gacha '
            'record. Please check your account and try again.',
        )
        raise ValueError('Cannot deduce uid from records')
    manager = GachaDataManager(uid)
    logger.info(f'Successfully connected to cache of uid {uid}')
    manager.log_stats()

    for gacha_type in GachaType:
        manager.add_records(gacha_type.value, record_cache[gacha_type.value])
        manager.gacha[gacha_type.value].sort()

    fileio.export_as_sql(manager, manager.cache_path)

    account_record.update_timestamp(uid)
    manager.log_stats()

    export_hooks = dict(
        csv=fileio.export_as_csv,
        html=fileio.export_as_html,
        json=fileio.export_as_json,
        md=fileio.export_as_md,
        xlsx=fileio.export_as_xlsx,
    )
    if 'all' in export:
        export = ['csv', 'html', 'json', 'md', 'srgf', 'xlsx']

    timestamp = time.strftime('%Y%m%d%H%M%S')

    for format in export:
        if format == 'srgf':
            format = 'srgf.json'
        logger.info(f'Exporting gacha data as {format} format')
        output_dir = os.getcwd()
        filename = f'HKSR-export-{uid}-{timestamp}.{format}'
        path = os.path.join(output_dir, filename)
        export_hooks[format](manager, path)
        logger.info(f'Gacha data in {format} format is saved to {path}')



import copy
import os
from typing import Dict

from prettytable import PrettyTable

from starrail.config import configuration as cfg
from starrail.gacha.database import DatabaseFactory
from starrail.gacha.type import GachaType
from starrail.utils import loggings

logger = loggings.get_logger(__file__)


class GachaDataList:
    def __init__(self, name, iterable=[], hash_key='id'):
        self.name = name
        self.data = []
        self.hash = set()
        self.hash_key = hash_key
        self.extend(iterable)

    def __len__(self):
        return len(self.data)

    def __getitem__(self, index):
        return self.data[index]

    def append(self, item):
        if item[self.hash_key] not in self.hash:
            self.data.append(item)
            self.hash.add(item[self.hash_key])
            return True
        return False

    def extend(self, iterable):
        return [self.append(item) for item in iterable]

    def sort(self):
        self.data.sort(key=lambda x: -int(x[self.hash_key]))

    def tolist(self):
        return copy.deepcopy(self.data)

    @property
    def stats(self):
        data = []
        for rank_type in ['5', '4', '3']:
            data.append(self.substats(rank_type))
        return data

    def substats(self, rank_type):
        count = sum([
            1 if item['rank_type'] ==
            rank_type else 0 for item in self.data
        ])
        total_items = len(self.data)
        if not total_items or not count:
            return dict(
                rank_type=rank_type,
                count='0',
                basic_prob='',
                compr_prob='',
                since_last=f'{total_items}',
                attempts=[],
                average='',
            )
        if rank_type == '3':
            return dict(
                rank_type=rank_type,
                count=f'{count}',
                basic_prob=f'{count / total_items:.2%}',
                compr_prob='',
                since_last='',
                attempts=[],
                average='',
            )
        attempts = []
        current_stats, current_attempts = None, 0
        for item in self.data + [dict(rank_type=rank_type, name='dummy')]:
            if item['rank_type'] == rank_type:
                attempts.append((current_stats, current_attempts))
                current_stats, current_attempts = item['name'], 0
            current_attempts += 1
        since_last = attempts[0][1]
        attempts = [f'{name}@{times}' for (name, times) in attempts]
        return dict(
            rank_type=rank_type,
            count=f'{count}',
            basic_prob=f'{count / total_items:.2%}',
            compr_prob=f'{count / (total_items - since_last):.2%}',
            since_last=f'{since_last}',
            attempts=attempts[1:],
            average=f'{(total_items - since_last) / count:.2f}',
        )


class GachaDataManager:

    def __init__(self, uid):
        self.uid: str = uid
        self.gacha = self.load_cache(uid)

    def load_cache(self, uid: str):
        db_dir = cfg.db_dir
        self.cache_path = os.path.join(db_dir, f'{uid}.sqlite3')
        if os.path.isfile(self.cache_path):
            return parse_cache_from_sql(self.cache_path)
        return init_empty_gacha_record()

    def log_stats(self):
        # a simple version
        fileds = ['Type', 'Count', 'Basic Prob.', ' True Prob.', 'Since Last']
        stats_string = f'*** Gacha stats for user uid = {self.uid}:\n'
        for gacha_type in GachaType:
            stats_string += f'  * {gacha_type.name}\n'
            stats = self.gacha[gacha_type.value].stats
            table = PrettyTable(field_names=fileds)
            for item in stats:
                table.add_row([
                    item['rank_type'], item['count'],
                    item['basic_prob'], item['compr_prob'],
                    item['since_last'],
                ])
            pretty = table.get_string()
            stats_string += f'{pretty}\n'
            if stats[0]['attempts']:
                attempt_string = ' '.join(stats[0]['attempts'])
                average = stats[0]['average']
                stats_string += ' History of 5-star gacha attempts: '
                stats_string += attempt_string
                stats_string += f'\n Average gacha per 5-star: {average}\n'
            stats_string += '\n'

        logger.info(stats_string)

    def add_records(self, gacha_id, records):
        r = self.gacha[gacha_id].extend(records)
        for success, record in zip(r, records):
            record['existing'] = not success


def parse_cache_from_sql(cache_path: str) -> Dict[int, GachaDataList]:
    cache = dict()
    with DatabaseFactory.get_database(cache_path) as db:
        for gacha_type in GachaType:
            entries = db.get_entries(gacha_type.name)
            data_list = GachaDataList(gacha_type.name, entries)
            cache[gacha_type.value] = data_list
    return cache


def init_empty_gacha_record() -> Dict[int, GachaDataList]:
    return {gt.value: GachaDataList(gt.name) for gt in GachaType}



import json
import platform
import traceback

from starrail.utils import loggings
from starrail.utils.misc import lazy_import

logger = loggings.get_logger(__file__)


fps_reg_key_cn = 'Software\\miHoYo\\崩坏：星穹铁道'
graphics_model = 'GraphicsSettings_Model_h2986158309'


def get_fps():
    if platform.system() == 'Windows':
        winreg = lazy_import('winreg')
        with winreg.OpenKey(
            winreg.HKEY_CURRENT_USER,
            fps_reg_key_cn,
            0,
            winreg.KEY_READ,
        ) as rk:
            settings = winreg.QueryValueEx(rk, graphics_model)
            settings = settings[0].decode('utf-8').strip('\x00')
            settings = json.loads(settings)
            return settings['FPS']
    else:
        logger.error(
            'Getting FPS is only supported on Windows platform, aborting.',
        )
        return 60


def safe_get_fps(default_value=60):
    try:
        return get_fps()
    except Exception:
        return default_value


def set_fps(value):
    if platform.system() == 'Windows':
        logger.info(f'trying to set fps to {value}')
        winreg = lazy_import('winreg')
        with winreg.OpenKey(
            winreg.HKEY_CURRENT_USER,
            fps_reg_key_cn,
            0,
            winreg.KEY_ALL_ACCESS,
        ) as rk:
            settings = winreg.QueryValueEx(rk, graphics_model)
            settings = settings[0].decode('utf-8').strip('\x00')
            settings = json.loads(settings)
            settings['FPS'] = value
            settings = json.dumps(settings) + '\x00'
            settings = settings.encode('utf-8')
            winreg.SetValueEx(
                rk,
                graphics_model,
                0,
                winreg.REG_BINARY,
                settings,
            )
        logger.info(f'Successfully set FPS to {value}.')
        return True, 'ok'
    else:
        logger.error(
            'Setting FPS is only supported on Windows platform, aborting.',
        )
        return False, 'Platform not supported'


def safe_set_fps(value):
    try:
        return set_fps(value)
    except Exception:
        logger.fatal(
            'Setting FPS is not success. Please first set FPS in '
            'game for one time and try again.',
        )
        return False, traceback.format_exc()



"""Config for graphic mode used by starrail package.

This configurataion overrides starrail.config, and should be compatible with
that one. The global configuration is read-only in graphic mode.
"""

import os
from enum import Enum
from pathlib import Path

import qfluentwidgets as qfw
from PySide6.QtCore import QLocale

userroot = os.path.abspath(os.path.expanduser('~'))


class Language(Enum):

    AUTO = QLocale()
    CHINESE_SIMPLIFIED = QLocale(
        QLocale.Language.Chinese,
        QLocale.Country.China,
    )
    ENGLISH = QLocale(
        QLocale.Language.English,
        QLocale.Country.UnitedStates,
    )


class LanguageSerializer(qfw.ConfigSerializer):

    def serialize(self, language: Language):
        return language.value.name() if language != Language.AUTO else 'auto'

    def deserialize(self, value):
        return Language(QLocale(value)) if value != 'auto' else Language.AUTO


class StarRailConfig(qfw.QConfig):

    cache_dir = os.path.join(userroot, '.starrail')
    db_dir = os.path.join(userroot, '.starrail', 'database')
    config_path = os.path.join(userroot, '.starrail', 'qconfig.json')

    locale = qfw.OptionsConfigItem(
        group='StarRailToolkit',
        name='Locale',
        default=Language.AUTO,
        validator=qfw.OptionsValidator(Language),
        serializer=LanguageSerializer(),
        restart=True,
    )

    check_update = qfw.ConfigItem(
        group='StarRailToolkit',
        name='CheckUpdate',
        default=True,
        validator=qfw.BoolValidator(),
        restart=False,
    )

    def __init__(self, path):
        super().__init__()
        self.file = Path(path)


qcfg = StarRailConfig(path=StarRailConfig.config_path)
qfw.qconfig.load(StarRailConfig.config_path, qcfg)
qcfg.set(qcfg.themeColor, '#ff0077dd')



import starrail.utils.babelfish.dictionary as dictionary
from starrail.utils.babelfish.translate import translate

from . import constants

average_gacha_per_5_star = dictionary.average_gacha_per_5_star
gacha_report = dictionary.gacha_report
gacha_title = dictionary.gacha_title
history_of_5_stars = dictionary.history_of_5_stars
html_thead = dictionary.html_thead
markdown_thead = dictionary.markdown_thead
ui_about = dictionary.ui_about
ui_about_this = dictionary.ui_about_this
ui_auto = dictionary.ui_auto
ui_cancel_update = dictionary.ui_cancel_update
ui_check_update = dictionary.ui_check_update
ui_check_update_at_start = dictionary.ui_check_update_at_start
ui_check_update_at_start_desc = dictionary.ui_check_update_at_start_desc
ui_check_update_fail = dictionary.ui_check_update_fail
ui_check_update_success = dictionary.ui_check_update_success
ui_copyright = dictionary.ui_copyright
ui_curr_fps = dictionary.ui_curr_fps
ui_current_version = dictionary.ui_current_version
ui_deduce_uid_fail = dictionary.ui_deduce_uid_fail
ui_downloading_gacha = dictionary.ui_downloading_gacha
ui_en = dictionary.ui_en
ui_extract_api_fail = dictionary.ui_extract_api_fail
ui_extract_api_fail_with_msg = dictionary.ui_extract_api_fail_with_msg
ui_extracting_api_url = dictionary.ui_extracting_api_url
ui_fine = dictionary.ui_fine
ui_fine1 = dictionary.ui_fine1
ui_fine2 = dictionary.ui_fine2
ui_fps_is_already = dictionary.ui_fps_is_already
ui_fps_is_set_to = dictionary.ui_fps_is_set_to
ui_gacha_basic_prob = dictionary.ui_gacha_basic_prob
ui_gacha_count = dictionary.ui_gacha_count
ui_gacha_since_last = dictionary.ui_gacha_since_last
ui_gacha_sync = dictionary.ui_gacha_sync
ui_gacha_sync_desc = dictionary.ui_gacha_sync_desc
ui_gacha_true_prob = dictionary.ui_gacha_true_prob
ui_gacha_type = dictionary.ui_gacha_type
ui_get_started = dictionary.ui_get_started
ui_get_started_desc = dictionary.ui_get_started_desc
ui_github_repo = dictionary.ui_github_repo
ui_github_repo_desc = dictionary.ui_github_repo_desc
ui_is_latest_version = dictionary.ui_is_latest_version
ui_locale = dictionary.ui_locale
ui_locale_setting_desc = dictionary.ui_locale_setting_desc
ui_no_data = dictionary.ui_no_data
ui_not_good = dictionary.ui_not_good
ui_ooops = dictionary.ui_ooops
ui_open_docs = dictionary.ui_open_docs
ui_open_issues = dictionary.ui_open_issues
ui_open_repo = dictionary.ui_open_repo
ui_open_troubleshooting = dictionary.ui_open_troubleshooting
ui_operation_zone = dictionary.ui_operation_zone
ui_personalization = dictionary.ui_personalization
ui_reset_fps = dictionary.ui_reset_fps
ui_reset_panel = dictionary.ui_reset_panel
ui_save_data = dictionary.ui_save_data
ui_save_success = dictionary.ui_save_success
ui_save_success_msg = dictionary.ui_save_success_msg
ui_send_feedback = dictionary.ui_send_feedback
ui_send_feedback_desc = dictionary.ui_send_feedback_desc
ui_set_fps_fail_with_msg = dictionary.ui_set_fps_fail_with_msg
ui_setting_failure = dictionary.ui_setting_failure
ui_setting_panel = dictionary.ui_setting_panel
ui_setting_restart_content = dictionary.ui_setting_restart_content
ui_setting_restart_title = dictionary.ui_setting_restart_title
ui_setting_success = dictionary.ui_setting_success
ui_settings = dictionary.ui_settings
ui_status_panel = dictionary.ui_status_panel
ui_sync = dictionary.ui_sync
ui_sync_gacha_fail = dictionary.ui_sync_gacha_fail
ui_sync_gacha_initial = dictionary.ui_sync_gacha_initial
ui_sync_gacha_success = dictionary.ui_sync_gacha_success
ui_synchronizing_gacha = dictionary.ui_synchronizing_gacha
ui_theme_mode = dictionary.ui_theme_mode
ui_theme_mode_dark = dictionary.ui_theme_mode_dark
ui_theme_mode_light = dictionary.ui_theme_mode_light
ui_theme_mode_settings_desc = dictionary.ui_theme_mode_settings_desc
ui_title = dictionary.ui_title
ui_traceback = dictionary.ui_traceback
ui_troubleshooting = dictionary.ui_troubleshooting
ui_troubleshooting_desc = dictionary.ui_troubleshooting_desc
ui_unlock_fps = dictionary.ui_unlock_fps
ui_unlock_fps_desc = dictionary.ui_unlock_fps_desc
ui_update_available = dictionary.ui_update_available
ui_update_desc = dictionary.ui_update_desc
ui_users = dictionary.ui_users
ui_users_desc = dictionary.ui_users_desc
ui_welcome = dictionary.ui_welcome
ui_zhs = dictionary.ui_zhs

__all__ = ['constants', 'translate']



import qfluentwidgets as qfw
from PySide6.QtCore import Qt, QThread, Signal
from PySide6.QtWidgets import QLabel

from starrail.gui.common.icon import Icon
from starrail.gui.interfaces.base import BaseInterface
from starrail.unlock.fps import safe_get_fps, safe_set_fps
from starrail.unlock.service import default_fps
from starrail.utils import babelfish


class SetFpsThread(QThread):

    successSignal = Signal(str)
    failureSignal = Signal(str)

    def __init__(self, value, parent=None):
        super().__init__(parent)
        self.value = value

    def run(self):
        success, msg = safe_set_fps(self.value)
        if success:
            self.successSignal.emit(babelfish.ui_fps_is_set_to(self.value))
        else:
            self.failureSignal.emit(babelfish.ui_set_fps_fail_with_msg(msg))


class UnlockFpsInterface(BaseInterface):

    def __init__(self, title, subtitle, parent):
        super().__init__(title, subtitle, parent)
        self.settingCard = self.addCard(babelfish.ui_setting_panel())
        self.statusCard = self.addCard(babelfish.ui_status_panel())

        self.spinBox = qfw.SpinBox()
        self.settingButton = qfw.PrimaryPushButton(
            text=babelfish.ui_unlock_fps(),
            parent=self.settingCard,
            icon=Icon.UNLOCK,
        )
        self.settingButton.clicked.connect(self.__setFpsSlot)
        self.resetButton = qfw.PrimaryPushButton(
            text=babelfish.ui_reset_fps(),
            parent=self.settingCard,
            icon=qfw.FluentIcon.CANCEL,
        )
        self.resetButton.clicked.connect(self.__resetFpsSlot)
        self.currFpsLabel = QLabel(text=babelfish.ui_curr_fps(''))
        self.currFps = 60

        self.setFpsThread = None

        self.__initWidget()
        self.__initStatus()

    def __initWidget(self):
        self.spinBox.setRange(30, 240)
        self.spinBox.setSingleStep(30)
        self.spinBox.setValue(120)
        self.settingCard.addWidget(self.spinBox)
        self.settingCard.addSpacing(10)
        self.settingCard.addWidget(self.settingButton)
        self.settingCard.addSpacing(10)
        self.settingCard.addWidget(self.resetButton)
        self.settingCard.addStretch(1)

        self.statusCard.addWidget(self.currFpsLabel)
        self.statusCard.addStretch(1)

    def __initStatus(self):
        self.currFps = safe_get_fps()
        self.__setCurrFPS(self.currFps)

    def __setCurrFPS(self, fps: int):
        self.currFps = fps
        self.currFpsLabel.setText(babelfish.ui_curr_fps(fps))

    def __setFps(self, value):
        if value == self.currFps:
            qfw.InfoBar.success(
                title=babelfish.ui_setting_success(),
                content=babelfish.ui_fps_is_already(value),
                orient=Qt.Orientation.Horizontal,
                isClosable=True,
                duration=2000,
                position=qfw.InfoBarPosition.TOP_RIGHT,
                parent=self,
            )
        else:
            self.setFpsThread = SetFpsThread(value, self)
            self.setFpsThread.successSignal.connect(self.__setSuccessSlot)
            self.setFpsThread.failureSignal.connect(self.__setFailureSlot)
            self.settingButton.setDisabled(True)
            self.resetButton.setDisabled(True)
            self.setFpsThread.start()

    def __setFpsSlot(self):
        targetFps = self.spinBox.value()
        self.__setFps(targetFps)

    def __resetFpsSlot(self):
        targetFps = default_fps
        self.__setFps(targetFps)

    def __setSuccessSlot(self, msg: str):
        self.__setCurrFPS(self.setFpsThread.value)
        self.settingButton.setEnabled(True)
        self.resetButton.setEnabled(True)
        self.setFpsThread = None

        qfw.InfoBar.success(
            title=babelfish.ui_setting_success(),
            content=msg,
            orient=Qt.Orientation.Horizontal,
            isClosable=True,
            duration=3000,
            position=qfw.InfoBarPosition.TOP_RIGHT,
            parent=self,
        )

    def __setFailureSlot(self, msg: str):
        self.settingButton.setEnabled(True)
        self.resetButton.setEnabled(True)
        self.setFpsThread = None

        qfw.InfoBar.error(
            title=babelfish.ui_setting_failure(),
            content=msg,
            orient=Qt.Orientation.Horizontal,
            isClosable=True,
            duration=-1,
            position=qfw.InfoBarPosition.TOP_RIGHT,
            parent=self,
        )



import json
import os
import time
import traceback
from typing import List

import qfluentwidgets as qfw
from PySide6 import QtWidgets
from PySide6.QtCore import Qt, QThread, Signal
from PySide6.QtWidgets import QLabel, QTableWidgetItem, QVBoxLayout

import starrail.gacha.service as service
from starrail.gacha.type import GachaType
from starrail.gui.common.stylesheet import StyleSheet
from starrail.gui.interfaces.base import BaseInterface
from starrail.utils import babelfish, loggings
from starrail.utils.accounts import account_record, get_latest_uid

AF = Qt.AlignmentFlag
logger = loggings.get_logger(__file__)


class GachaSyncThread(QThread):

    syncStateSignal = Signal(str)
    syncSuccessSignal = Signal(str)
    syncFailSignal = Signal(str)

    def __init__(self, parent=None):
        super().__init__(parent=parent)

    def run(self):
        try:
            uid = self.syncGachaData()
            self.syncSuccessSignal.emit(uid)
        except Exception:
            self.syncFailSignal.emit(
                f'{babelfish.ui_traceback()}:\n{traceback.format_exc()}',
            )

    def logAndUpdateState(self, message, level=loggings.logging.INFO):
        logger.log(level=level, msg=f'[GUI] {message}')
        self.syncStateSignal.emit(message)

    def exportGachaType(self, api_template, gacha_type, request_interval):
        r = []
        end_id = '0'
        for page in service.integers():
            self.logAndUpdateState(
                babelfish.ui_downloading_gacha(
                    name=babelfish.translate(gacha_type.name),
                    page=page,
                ),
            )
            api_url = service.get_api_url(
                api_template, end_id, str(gacha_type.value),
                str(page), '5',
            )
            logger.debug(f'Requesting {api_url}')
            response, code = service.fetch_json(api_url)
            time.sleep(request_interval)
            _, should_stop, msg = service.check_response(response, code)
            logger.info(f'check_response: {msg}')
            if should_stop:
                break
            data_list = response['data']['list']
            r.extend(data_list)
            end_id = data_list[-1]['id']
        return r

    def syncGachaData(self):
        self.logAndUpdateState(babelfish.ui_extracting_api_url())
        api_url = service.detect_api_url()
        response, code = service.fetch_json(api_url)
        valid, _, msg = service.check_response(response, code)
        logger.info(f'check_response (api): {msg}')
        if not valid:
            self.syncFailSignal.emit(babelfish.ui_extract_api_fail())
            raise ValueError(babelfish.ui_extract_api_fail_with_msg(msg))

        api_template = service.get_url_template(api_url)
        record_cache = dict()

        for gacha_type in service.GachaType:
            records = self.exportGachaType(
                api_template,
                gacha_type,
                0.15,
            )
            record_cache[gacha_type.value] = records
            logger.info(f'Finish downloading records of {gacha_type.name}')

        uid = service.deduce_uid(record_cache)
        if not uid:
            logger.critical(
                'Cannot deduce uid from records, there may be no '
                'record. Please check and try again.',
            )
            raise ValueError(babelfish.ui_deduce_uid_fail())
        manager = service.GachaDataManager(uid)
        logger.info(f'Successfully connected to cache of uid {uid}')
        manager.log_stats()

        for gacha_type in service.GachaType:
            manager.add_records(
                gacha_type.value, record_cache[gacha_type.value],
            )
            manager.gacha[gacha_type.value].sort()

        service.fileio.export_as_sql(manager, manager.cache_path)
        manager.log_stats()

        return uid


class RecordExportThread(QThread):

    saveSuccessSignal = Signal(str)

    def __init__(self, uid, path, parent=None):
        super().__init__(parent=parent)
        self.uid = uid
        self.path = path

    def run(self):
        manager = service.GachaDataManager(self.uid)
        export_hooks = dict(
            csv=service.fileio.export_as_csv,
            html=service.fileio.export_as_html,
            json=service.fileio.export_as_json,
            md=service.fileio.export_as_md,
            srgf=service.fileio.export_as_srgf,
            xlsx=service.fileio.export_as_xlsx,
        )
        timestamp = time.strftime('%Y%m%d%H%M%S')
        for format, hook in export_hooks.items():
            if format == 'srgf':
                format = 'srgf.json'
            filename = f'HKSR-export-{self.uid}-{timestamp}.{format}'
            export_path = os.path.join(self.path, filename)
            hook(manager, export_path)
        self.saveSuccessSignal.emit(self.path)


class ResultTableWidget(QtWidgets.QWidget):

    def __init__(self, title, parent=None):
        super().__init__(parent=parent)
        self.vBoxLayout = QVBoxLayout(self)
        self.labelsLayout = qfw.FlowLayout()

        self.titleLabel = QLabel(title)
        self.table = qfw.TableWidget(self)

        self.__initWidgets()
        self.__initTable()

    def __initWidgets(self):
        self.table.verticalHeader().hide()
        self.table.setColumnCount(5)
        self.table.setRowCount(3)
        self.table.setHorizontalHeaderLabels([
            babelfish.ui_gacha_type(), babelfish.ui_gacha_count(),
            babelfish.ui_gacha_basic_prob(), babelfish.ui_gacha_true_prob(),
            babelfish.ui_gacha_since_last(),
        ])
        self.table.setEnabled(False)
        SM = QtWidgets.QAbstractItemView.ScrollMode
        self.table.setHorizontalScrollMode(SM.ScrollPerPixel)
        self.table.setVerticalScrollMode(SM.ScrollPerPixel)

        self.labelsLayout.setContentsMargins(0, 0, 0, 10)
        self.labelsLayout.setHorizontalSpacing(10)
        self.labelsLayout.setVerticalSpacing(20)

        self.vBoxLayout.setContentsMargins(10, 10, 0, 0)
        self.vBoxLayout.addWidget(self.titleLabel)
        self.vBoxLayout.addWidget(self.table)
        self.vBoxLayout.addLayout(self.labelsLayout)

        self.setFixedWidth(580)
        self.setObjectName('frame')
        self.titleLabel.setObjectName('titleLabel')
        self.table.setObjectName('table')
        StyleSheet.RESULT_TABLE_WIDGET.apply(self)

    def __initTable(self):
        self.setTableData([
            ['5'] + [babelfish.ui_no_data()] * 4,
            ['4'] + [babelfish.ui_no_data()] * 4,
            ['3'] + [babelfish.ui_no_data()] * 4,
        ])

    def setTableData(self, data: List[List[str]]):
        # data: 3x5
        logger.debug(f'[GUI] {data}')
        for idx0, row in enumerate(data):
            for idx1, item in enumerate(row):
                self.table.setItem(idx0, idx1, QTableWidgetItem(item))


class GachaSyncInterface(BaseInterface):

    def __init__(self, title, subtitle, parent):
        super().__init__(title, subtitle, parent)

        # Buttons
        self.buttonsCard = None
        self.syncButton = qfw.PrimaryPushButton(
            text=babelfish.ui_sync(),
            parent=self,
            icon=qfw.FluentIcon.SYNC,
        )
        self.syncButton.clicked.connect(self.onSyncButtonClicked)
        self.saveButton = qfw.PrimaryPushButton(
            text=babelfish.ui_save_data(),
            parent=self,
            icon=qfw.FluentIcon.SAVE_AS,
        )
        self.saveButton.clicked.connect(self.onSaveButtonClicked)
        self.saveButton.setDisabled(True)

        # Tables
        self.tableLayout = qfw.FlowLayout()
        self.stellarTable = ResultTableWidget(
            babelfish.translate('STELLAR'),
            self,
        )
        self.departureTable = ResultTableWidget(
            babelfish.translate('DEPARTURE'),
            self,
        )
        self.characterTable = ResultTableWidget(
            babelfish.translate('CHARACTER'),
            self,
        )
        self.lightConeTable = ResultTableWidget(
            babelfish.translate('LIGHT_CONE'),
            self,
        )

        # Gacha Sync Temp Objects
        self.syncThread = None
        self.syncToolTip = None
        self.saveThread = None

        self.uid = get_latest_uid()
        logger.info(f'Detected current uid: {self.uid}')
        if self.uid:
            self.updateTableDisplay()
            self.saveButton.setEnabled(True)

        self.__initWidget()

    def __initWidget(self):
        self.buttonsCard = self.addCard(babelfish.ui_operation_zone())
        self.buttonsCard.addWidget(self.syncButton)
        self.buttonsCard.addSpacing(10)
        self.buttonsCard.addWidget(self.saveButton)
        self.buttonsCard.addStretch(1)

        self.tableLayout.addWidget(self.stellarTable)
        self.tableLayout.addWidget(self.departureTable)
        self.tableLayout.addWidget(self.characterTable)
        self.tableLayout.addWidget(self.lightConeTable)

        self.vBoxLayout.addLayout(self.tableLayout)

    def resizeEvent(self, e):
        if self.syncToolTip:
            self.syncToolTip.move(self.syncToolTip.getSuitablePos())
        return super().resizeEvent(e)

    def updateTableDisplay(self):
        manager = service.GachaDataManager(self.uid)
        mapping = {
            GachaType.STELLAR.name: self.stellarTable,
            GachaType.DEPARTURE.name: self.departureTable,
            GachaType.CHARACTER.name: self.characterTable,
            GachaType.LIGHT_CONE.name: self.lightConeTable,
        }
        for gacha_type in GachaType:
            stats = manager.gacha[gacha_type.value].stats
            table = [
                [
                    item['rank_type'], item['count'],
                    item['basic_prob'], item['compr_prob'],
                    item['since_last'],
                ] for item in stats
            ]
            mapping[gacha_type.name].setTableData(table)

    def onSyncButtonClicked(self):
        logger.info('[GUI] Start gacha data synchronization')

        self.syncButton.setDisabled(True)
        self.saveButton.setDisabled(True)

        self.syncToolTip = qfw.StateToolTip(
            babelfish.ui_synchronizing_gacha(),
            babelfish.ui_sync_gacha_initial(),
            self.window(),
        )
        self.syncToolTip.move(self.syncToolTip.getSuitablePos())
        self.syncToolTip.show()

        self.syncThread = GachaSyncThread(self)
        self.syncThread.syncStateSignal.connect(
            lambda s: self.setToolTipContentSlot(self.syncToolTip, s),
        )
        self.syncThread.syncSuccessSignal.connect(self.syncSuccessSlot)
        self.syncThread.syncFailSignal.connect(self.syncFailSlot)
        self.syncThread.start()

    def onSaveButtonClicked(self):
        logger.info('[GUI] Start gacha data exporting')

        self.syncButton.setDisabled(True)
        self.saveButton.setDisabled(True)

        path = QtWidgets.QFileDialog.getExistingDirectory(
            self, 'Select Export Folder', os.path.expanduser('~'),
        )
        if path:
            self.saveThread = RecordExportThread(self.uid, path, self)
            self.saveThread.saveSuccessSignal.connect(self.saveSuccessSlot)
            self.saveThread.start()
        else:
            self.syncButton.setEnabled(True)
            self.saveButton.setEnabled(True)

    def onSyncSuccess(self):
        self.updateTableDisplay()

    def setToolTipContentSlot(self, tooltip: qfw.StateToolTip, content: str):
        tooltip.setContent(content)

    def syncSuccessSlot(self, uid: int):
        logger.info('[GUI] Gacha data synchronization success')

        self.uid = uid
        account_record.update_timestamp(uid)

        self.syncToolTip.setTitle(babelfish.ui_sync_gacha_success())
        self.syncToolTip.setContent('')
        self.syncToolTip.setState(True)
        self.syncToolTip = None
        self.syncThread = None

        self.syncButton.setEnabled(True)
        self.saveButton.setEnabled(True)

        self.onSyncSuccess()

    def syncFailSlot(self, message: str):
        logger.info(f'[GUI] Gacha data sync fail with message {message}')

        if self.syncToolTip is not None:
            self.syncToolTip.setTitle(babelfish.ui_sync_gacha_fail())
            self.syncToolTip.setContent('')
            self.syncToolTip.setState(True)
            self.syncToolTip = None
            self.syncThread = None

        qfw.InfoBar.error(
            title=babelfish.ui_sync_gacha_fail(),
            content=message,
            orient=Qt.Orientation.Horizontal,
            isClosable=True,
            duration=-1,
            position=qfw.InfoBarPosition.TOP_RIGHT,
            parent=self,
        )

        self.syncButton.setEnabled(True)
        self.saveButton.setEnabled(True)

    def tableDisplaySlot(self, data):
        mapping = {
            GachaType.STELLAR.name: self.stellarTable,
            GachaType.DEPARTURE.name: self.departureTable,
            GachaType.CHARACTER.name: self.characterTable,
            GachaType.LIGHT_CONE.name: self.lightConeTable,
        }
        for key, value in json.loads(data).items():
            mapping[key].setTableData(value)

    def saveSuccessSlot(self, path):
        self.saveThread = None

        qfw.InfoBar.success(
            title=babelfish.ui_save_success(),
            content=babelfish.ui_save_success_msg(path),
            orient=Qt.Orientation.Horizontal,
            isClosable=True,
            duration=-1,
            position=qfw.InfoBarPosition.TOP_RIGHT,
            parent=self,
        )

        self.syncButton.setEnabled(True)
        self.saveButton.setEnabled(True)



from enum import Enum

from qfluentwidgets import FluentIconBase, Theme, getIconColor


class Icon(FluentIconBase, Enum):

    UNLOCK = 'Unlock'
    USER = 'User'

    def path(self, theme=Theme.AUTO):
        return f'resources/images/icons/{self.value}_{getIconColor(theme)}.svg'



import argparse
import traceback

from starrail import __version__, digital_version
from starrail.config import configuration as cfg
from starrail.entry.setup import setup
from starrail.gacha.service import export_gacha_from_api
from starrail.unlock.service import unlock_fps
from starrail.utils import babelfish, loggings
from starrail.utils.auto_update import check_update

logger = loggings.get_logger(__file__)


def get_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog='hksr',
        description='Honkai: Star Rail Toolkit',
    )
    parser.add_argument(
        '--locale', type=str, default='zhs',
        choices=['en', 'zhs'],
        help=(
            'Language of gacha report. Abbreviations: `en` for English, '
            '`zhs` for simplified Chinese'
        ),
    )
    parser.add_argument(
        '--log-level', type=str, default='DEBUG',
        choices=['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG'],
        help='Controlling the level of logging output.',
    )

    subparsers = parser.add_subparsers(dest='command')

    gacha = subparsers.add_parser('gacha')
    gacha.add_argument(
        '--api', type=str,
        help='URL of the gacha api, please refer to README.md for details.',
    )
    gacha.add_argument(
        '--export', nargs='+', type=str, default=['all'],
        choices=['all', 'csv', 'html', 'json', 'md', 'srgf', 'xlsx'],
        help='Types of expected export formats.',
    )
    gacha.add_argument(
        '--request-interval', type=float, default=0.1,
        help='Minimum interval (seconds) between two requests.',
    )

    unlock = subparsers.add_parser('unlock')
    unlock.add_argument(
        '--fps', type=int,
        help='Target FPS of the game',
    )
    unlock.add_argument(
        '--reset', action='store_true',
        help='Reset FPS to the initial value',
    )

    return parser


def cli_check_update():
    if cfg.check_update:
        try:
            latest = check_update()
            logger.info('Check Update:')
            logger.info(f' * Current version is {__version__}')
            logger.info(f' * Latest release version is {latest.version}')
            if digital_version(__version__) < digital_version(latest.version):
                cmd = 'pip install starrail-toolkit -U --force-reinstall'
                logger.info('New version of package is available.')
                logger.info(
                    f'If you use cli, please update with pip: `f{cmd}`',
                )
                logger.info('If you use GUI, please download executables:')
                for name, url in latest.dist.items():
                    logger.info(f' * {babelfish.translate(name)}: {url}')
            else:
                logger.info('Your program is the latest version.')
        except Exception:
            logger.warning(
                'Check update failed. '
                'Please check your network connection.',
            )


def cli_entry():
    parser = get_parser()
    args = parser.parse_args()
    setup(log_level=args.log_level, locale=args.locale)
    logger.info(args)
    logger.info(cfg)
    cli_check_update()

    if args.command == 'gacha':
        export_gacha_from_api(
            api_url=args.api,
            export=args.export,
            request_interval=args.request_interval,
        )
    elif args.command == 'unlock':
        unlock_fps(fps=args.fps, reset=args.reset)
    else:
        parser.print_help()


if __name__ == '__main__':
    try:
        cli_entry()
    except Exception:
        traceback.print_exc()
    finally:
        input('Press `Enter` to exit.')



from starrail.utils.babelfish.multilingual import MultilingualString as _MS

# Lookup table - used by dictionary

lookup_table = dict(
    STELLAR=_MS(en='Stellar Warp', zhs='群星跃迁'),
    DEPARTURE=_MS(en='Departure Warp', zhs='始发跃迁'),
    CHARACTER=_MS(en='Character Event Warp', zhs='角色跃迁'),
    LIGHT_CONE=_MS(en='Light Cone Event Warp', zhs='光锥跃迁'),
    GitHub_dist=_MS(en='GitHub Distribution', zhs='GitHub下载'),
    netdisk_dist=_MS(en='Net Disk Distribution', zhs='网盘下载'),
)

# === BEGIN OF PRE-DEFINED VOCABULARIES ===

average_gacha_per_5_star = _MS(
    en='Average gacha per 5-star',
    zhs='五星平均抽数',
)
gacha_report = _MS(en='Gacha Report', zhs='抽卡分析')
gacha_title = _MS(
    en='Gacha Report for Honkai: Star Rail Player {}',
    zhs='《崩坏：星穹铁道》玩家{}的抽卡分析',
)
history_of_5_stars = _MS(
    en='History of 5-star gacha attempts',
    zhs='五星出货记录',
)
html_thead = _MS(
    en=(
        '<th>Type</th><th>Count</th><th>Basic Prob.</th><th>True Prob.</th>'
        '<th>Since Last</th>'
    ),
    zhs=('<th>星级</th><th>数量</th><th>基础概率</th><th>综合概率</th><th>距上次</th>'),
)
markdown_thead = _MS(
    en=(
        '| Type | Count | Basic Prob. | True Prob. | Since Last |\n'
        '| ---- | ----- | ----------- | ---------- | ---------- |\n'
    ),
    zhs=(
        '| 星级 | 数量 | 基础概率 | 综合概率 | 距上次 |\n'
        '| ---- | ---- | ------ | ------- | ---- |\n'
    ),
)
ui_about = _MS(en='About', zhs='关于')
ui_about_this = _MS(en='About This Application', zhs='关于本软件')
ui_auto = _MS(en='Auto', zhs='跟随系统设置')
ui_cancel_update = _MS(en='Cancel Update', zhs='暂不更新')
ui_check_update = _MS(en='Check Update', zhs='检查更新')
ui_check_update_at_start = _MS(
    en='Check Update at Application Start',
    zhs='启动时自动检查更新',
)
ui_check_update_at_start_desc = _MS(
    en='It\'s strongly recommended to turn this on to get the latest feature',
    zhs='强烈建议打开检查更新，本app版本快速迭代中，会加入更多功能。',
)
ui_check_update_fail = _MS(
    en='Check update failed. Please check your network connection.',
    zhs='检查更新失败，请检查你的网络连接并重试。（不重试也行，但是你可能没联网）',
)
ui_check_update_success = _MS(
    en='Check update finished',
    zhs='检查更新完成',
)
ui_copyright = _MS(
    en='Copyright © {} LittleNyima.',
    zhs='版权所有 © {} LittleNyima。',
)
ui_curr_fps = _MS(
    en='Current FPS in Game Settings: {}.',
    zhs='当前游戏设定帧率：{}。',
)
ui_current_version = _MS(
    en='Current version: {}.',
    zhs='当前版本：{}。',
)
ui_deduce_uid_fail = _MS(
    en='Fail to deduce uid. There may be no record. Please check.',
    zhs='检测 UID 失败，账号可能没有抽卡记录，请重试。',
)
ui_downloading_gacha = _MS(
    en='Downloading page {page} of type {name}',
    zhs='正在导出{name}的第{page}页',
)
ui_en = _MS(
    en='English',
    zhs='英语',
)
ui_extract_api_fail = _MS(
    en='Extracting API URL failed. Please retry after exiting the game.',
    zhs='提取 API URL 失败，请关闭游戏或重新查询抽卡信息后重试。',
)
ui_extract_api_fail_with_msg = _MS(
    en='Extracting API URL failed. Failed with: {}',
    zhs='提取 API URL 失败，遇到的问题：{}',
)
ui_extracting_api_url = _MS(
    en='Extracting API URL',
    zhs='正在提取 API URL',
)
ui_fine = _MS(en='Fine', zhs='好')
ui_fine1 = _MS(en='Fine', zhs='好的')
ui_fine2 = _MS(en='Fine', zhs='好吧')
ui_fps_is_already = _MS(
    en='FPS is already {}~',
    zhs='帧率已经是{}啦～',
)
ui_fps_is_set_to = _MS(
    en='FPS is successfully set to {}.',
    zhs='帧率已成功设置为{}。',
)
ui_gacha_basic_prob = _MS(en='Basic Prob.', zhs='基础概率')
ui_gacha_count = _MS(en='Count', zhs='数量')
ui_gacha_since_last = _MS(en='Since Last', zhs='距上次')
ui_gacha_sync = _MS(en='Sync Gacha Data', zhs='抽卡数据同步')
ui_gacha_sync_desc = _MS(
    en=(
        'First query warp record in the game, then close the game and click '
        'the sync button.'
    ),
    zhs='先在游戏里查询一次抽卡记录，然后退出游戏，点击同步按钮。',
)
ui_gacha_true_prob = _MS(en='True Prob.', zhs='综合概率')
ui_gacha_type = _MS(en='Type', zhs='星级')
ui_get_started = _MS(en='Get Started', zhs='开始使用')
ui_get_started_desc = _MS(
    en='Learning how to use the toolkit.',
    zhs='学习本工具箱的基本使用方法。',
)
ui_github_repo = _MS(en='GitHub Repo', zhs='开源代码')
ui_github_repo_desc = _MS(
    en='This project is open-resource, licenced under GPLv3. View code here.',
    zhs='本项目代码开源，以 GPLv3 分发。点此查看源代码。',
)
ui_is_latest_version = _MS(
    en='Your application is the latest version.',
    zhs='已经是最新版本啦',
)
ui_locale = _MS(en='Display Language', zhs='显示语言')
ui_locale_setting_desc = _MS(en='Changing the display language', zhs='改变显示语言')
ui_no_data = _MS(en='No Data', zhs='暂无数据')
ui_not_good = _MS(en='No', zhs='不好')
ui_ooops = _MS(en='OOOPS!', zhs='出错了！')
ui_open_docs = _MS(en='Open Documentations', zhs='打开帮助页面')
ui_open_issues = _MS(en='Open Issue Page', zhs='打开反馈页面')
ui_open_repo = _MS(en='Open GitHub Repo', zhs='打开代码仓库')
ui_open_troubleshooting = _MS(en='Open Troubleshooting', zhs='打开常见问题')
ui_operation_zone = _MS(en='Operations', zhs='操作区')
ui_personalization = _MS(en='Personalization', zhs='个性化')
ui_reset_fps = _MS(
    en='Reset FPS',
    zhs='重置帧率',
)
ui_reset_panel = _MS(en='Reset Panel', zhs='重置面板')
ui_save_data = _MS(en='Save As', zhs='保存数据')
ui_save_success = _MS(en='Data Save Success', zhs='保存成功')
ui_save_success_msg = _MS(en='Data is saved to {}', zhs='数据已保存到{}')
ui_send_feedback = _MS(en='Send Feedback', zhs='提交反馈')
ui_send_feedback_desc = _MS(
    en='Feedback is welcome if you encounter problems or have suggestions.',
    zhs='如果你遇到问题，或对本项目有更多建议，欢迎提交反馈。',
)
ui_set_fps_fail_with_msg = _MS(
    en='Setting FPS failed. Failed with: {}',
    zhs='设置帧率失败，遇到的问题：{}',
)
ui_setting_failure = _MS(
    en='Error',
    zhs='设置失败',
)
ui_setting_panel = _MS(en='Setting Panel', zhs='设置面板')
ui_setting_restart_content = _MS(
    en='Please restart to apply changes.',
    zhs='请重启以应用新设置。',
)
ui_setting_restart_title = _MS(en='Success', zhs='设置成功')
ui_setting_success = _MS(en='Success', zhs='设置成功')
ui_settings = _MS(en='Settings', zhs='设置')
ui_status_panel = _MS(en='Status Panel', zhs='状态面板')
ui_sync = _MS(en='Sync Data', zhs='同步数据')
ui_sync_gacha_fail = _MS(
    en='Synchronization Failed',
    zhs='同步失败',
)
ui_sync_gacha_initial = _MS(
    en='Initializing...',
    zhs='正在初始化同步线程',
)
ui_sync_gacha_success = _MS(
    en='Synchronization Success',
    zhs='同步成功',
)
ui_synchronizing_gacha = _MS(
    en='Synchronizing...',
    zhs='正在同步抽卡数据',
)
ui_theme_mode = _MS(en='Theme Mode', zhs='应用主题')
ui_theme_mode_dark = _MS(en='Dark', zhs='深色')
ui_theme_mode_light = _MS(en='Light', zhs='浅色')
ui_theme_mode_settings_desc = _MS(
    en='Changing the appearance of application',
    zhs='改变应用外观',
)
ui_title = _MS(en='Honkai: Star Rail Toolkit', zhs='崩坏：星穹铁道工具箱')
ui_traceback = _MS(en='Traceback', zhs='报错信息')
ui_troubleshooting = _MS(en='TroubleShooting', zhs='常见问题')
ui_troubleshooting_desc = _MS(
    en='Click here to view FAQ.',
    zhs='点此查看常见问题解答。',
)
ui_unlock_fps = _MS(
    en='Unlock FPS Settings',
    zhs='解锁帧率',
)
ui_unlock_fps_desc = _MS(
    en=(
        'First set the frame rate in the game at least once, and then use the '
        'functions of this interface.'
    ),
    zhs='首先至少在游戏中设置一次帧率，再使用本页面中的功能。',
)
ui_update_available = _MS(
    en='New Version is Available',
    zhs='检查到新版本',
)
ui_update_desc = _MS(
    en=(
        'New Version of Honkai: Star Rail Toolkit is available. '
        'CHANGELOG: {}\n Please update to the latest version to experience '
        'the latest features. If the connectivity of GitHub is not good, '
        'it is recommended to download from the network disk distribution.'
    ),
    zhs=(
        '崩坏：星穹铁道工具箱有新的可用版本，更新日志：{}\n 请更新到最新版本以使用全新'
        '功能。如果你对GitHub的连接不稳定，请从网盘下载最新版本。'
    ),
)
ui_users = _MS(en='Users', zhs='用户管理')
ui_users_desc = _MS(en='Multi-user management', zhs='如果有多个账号可以用这个切换')
ui_welcome = _MS(
    en='Welcome to HKSR Toolkit!',
    zhs='欢迎登车！',
)
ui_zhs = _MS(
    en='Simplified Chinese',
    zhs='简体中文',
)

# === END OF PRE-DEFINED VOCABULARIES ===



import json
import os
import time

from starrail.config import configuration as cfg
from starrail.utils import babelfish


def todict(d, keys):
    return {k: d[k] for k in d if k in keys}


class AccountRecord:

    def __init__(self, path):
        self._path = path
        cache = self.safe_load()
        self.meta = cache['meta']
        self.accounts = cache['accounts']

    def safe_load(self):
        try:
            with open(self.path, encoding='utf-8') as f:
                cache = json.load(f)
            return cache
        except Exception:
            return dict(
                meta=dict(latest=''),
                accounts=dict(),
            )

    def flush(self):
        dirname = os.path.dirname(self._path)
        os.makedirs(dirname, exist_ok=True)
        with open(self._path, 'w', encoding='utf-8') as f:
            json.dump(
                dict(meta=self.meta, accounts=self.accounts), f,
                ensure_ascii=False, indent=2,
            )

    def update_timestamp(self, uid):
        timestamp = time.strftime(babelfish.constants.TIME_FMT)
        if uid in self.accounts:
            self.accounts[uid]['last_update'] = timestamp
        else:
            self.accounts[uid] = dict(last_update=timestamp)
        self.meta['latest'] = uid
        self.flush()

    def latest_uid(self):
        return self.meta['latest']


account_record = AccountRecord(cfg.account_record_path)


def get_latest_uid():
    if account_record.latest_uid():
        return account_record.latest_uid()
    if os.path.isdir(cfg.db_dir):
        caches = os.listdir(cfg.db_dir)
        caches = [
            cache.split('.')[0]
            for cache in caches if cache.endswith('.sqlite3')
        ]
        if caches:
            return sorted(caches)[0]
    return ''



from starrail.utils.loggings import get_logger

logger = get_logger(__file__)


class Locale:

    lang = 'zhs'

    available_lang = {'en', 'zhs'}

    default_lang = 'zhs'


locale_mapping = {
    'en_US': 'en',
    'zh_CN': 'zhs',
}


def setup_locale(lang):
    if lang in locale_mapping:
        lang = locale_mapping[lang]
    elif lang not in Locale.available_lang:
        logger.warn(
            f'{lang} is not an available language, which may cause '
            'unexpected outputs',
        )
        logger.warn(f'Locale falls back to {Locale.default_lang}')
    Locale.lang = lang



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel

from starrail.gui.interfaces.base import BaseInterface


class UsersInterface(BaseInterface):

    def __init__(self, title, subtitle, parent):
        super().__init__(title, subtitle, parent)
        self.vBoxLayout.addWidget(
            QLabel('coming soon'), 0, Qt.AlignmentFlag.AlignTop,
        )



REPO_URL = 'https://github.com/LittleNyima/honkai-starrail-toolkit'
DOCS_URL = f'{REPO_URL}/tree/develop/docs'
TROUBLESHOOTING_URL = f'{DOCS_URL}/troubleshooting.md'
ISSUE_URL = f'{REPO_URL}/issues'
TIME_FMT = '%Y-%m-%d_%H:%M:%S'



import sqlite3
from typing import Dict, List, Tuple

from starrail.gacha.type import GachaType
from starrail.utils import loggings

logger = loggings.get_logger(__file__)


class GachaDatabase:
    def __init__(self, db_name: str) -> None:
        """
        A class to handle SQLite database operations related to gacha.

        Args:
            db_name: A string representing the name of the SQLite database
                to use.
        """

        self.conn = sqlite3.connect(db_name)
        self.cursor = self.conn.cursor()
        for gacha_type in GachaType:
            self.create_table(gacha_type.name)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def create_table(self, table_name: str) -> None:
        """
        Creates a new table in the database with the specified name.

        Args:
            table_name: A string representing the name of the table to create.
        """

        self.cursor.execute(f'''CREATE TABLE IF NOT EXISTS {table_name} (
            uid TEXT,
            gacha_id TEXT,
            gacha_type TEXT,
            item_id TEXT,
            count TEXT,
            time TEXT,
            name TEXT,
            lang TEXT,
            item_type TEXT,
            rank_type TEXT,
            id TEXT PRIMARY KEY
        );''')
        self.conn.commit()

    def add_entry(self, table: str, entry: Dict[str, str]) -> None:
        """
        Adds a new entry to the specified table in the database.

        Args:
            table: A string representing the name of the table to add the
                entry to.
            entry: A dictionary representing the entry to add to the table.
        """

        self.cursor.execute(
            f'INSERT INTO {table} VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);',
            self.parse_entry(entry),
        )
        self.conn.commit()

    def get_entries(self, table: str) -> List[Dict[str, str]]:
        """
        Retrieves all entries from the specified table in the database.

        Args:
            table: A string representing the name of the table to retrieve
                entries from.

        Returns:
            A list of dictionaries representing the entries retrieved from the
                table.
        """

        self.cursor.execute(f'SELECT * FROM {table};')
        entries = self.cursor.fetchall()
        return [self.unparse_entry(*entry) for entry in entries]

    def close(self) -> None:
        """
        Closes the connection to the database.
        """

        self.cursor.close()
        self.conn.close()

    @property
    def version(self) -> int:
        """
        The version of the database, which should be identical to API_VERSION.

        Returns:
            An integer representing the version of the database.
        """

        self.cursor.execute('PRAGMA user_version;')
        entries = self.cursor.fetchone()[0]
        return entries

    @version.setter
    def version(self, value: int) -> None:
        """
        Sets the version of the database.

        Args:
            value: An integer representing the new version of the database.
        """

        self.cursor.execute(f'PRAGMA user_version={value};')
        self.conn.commit()

    @staticmethod
    def parse_entry(entry: Dict[str, str]) -> Tuple:
        """
        Parses an entry dictionary into a tuple for insertion into the
        database.

        Args:
            entry: A dictionary representing the entry to parse.

        Returns:
            A tuple representing the parsed entry.
        """

        uid = entry['uid']
        gacha_id = entry['gacha_id']
        gacha_type = entry['gacha_type']
        item_id = entry['item_id']
        count = entry['count']
        time_ = entry['time']
        name = entry['name']
        lang = entry['lang']
        item_type = entry['item_type']
        rank_type = entry['rank_type']
        id_ = entry['id']
        return (
            uid, gacha_id, gacha_type, item_id, count, time_, name, lang,
            item_type, rank_type, id_,
        )

    @staticmethod
    def unparse_entry(
        uid: str, gacha_id: str, gacha_type: str, item_id: str, count: str,
        time_: str, name: str, lang: str, item_type: str, rank_type: str,
        id_: str,
    ) -> Dict[str, str]:
        """
        Unparses a tuple from the database into an entry dictionary.

        Args:
            uid: A string representing the UID of the entry.
            gacha_id: A string representing the Gacha ID of the entry.
            gacha_type: A string representing the Gacha type of the entry.
            item_id: A string representing the Item ID of the entry.
            count: A string representing the count of the entry.
            time_: A string representing the timestamp of the entry.
            name: A string representing the name of the entry.
            lang: A string representing the language of the entry.
            item_type: A string representing the Item type of the entry.
            rank_type: A string representing the Rank type of the entry.
            id_: A string representing the ID of the entry.

        Returns:
            A dictionary representing the unparsed entry.
        """

        return dict(
            uid=uid, gacha_id=gacha_id, gacha_type=gacha_type,
            item_id=item_id, count=count, time=time_, name=name,
            lang=lang, item_type=item_type, rank_type=rank_type,
            id=id_,
        )


class DatabaseFactory:

    API_VERSION = 1

    @staticmethod
    def get_database(db_name: str) -> GachaDatabase:
        """
        Retrieves a new instance of a GachaDatabase with the specified name.

        Args:
            db_name: A string representing the name of the database to
                connect to.

        Returns:
            A new instance of a GachaDatabase.
        """
        db = GachaDatabase(db_name)
        if db.version == 0:
            db.version = DatabaseFactory.API_VERSION
        elif db.version < DatabaseFactory.API_VERSION:
            logger.warn(
                'Version of your database is lower than api version. '
                'Migration is required or potential data loss will '
                'occur.',
            )
        elif db.version > DatabaseFactory.API_VERSION:
            logger.warn(
                'Version of the program is lower than database '
                'version. Please update your program or unexpected '
                'errors will occur.',
            )
        return db



import copy
from typing import Dict

import qfluentwidgets as qfw
from PySide6 import QtWidgets
from PySide6.QtCore import QEvent, Qt, QUrl, Signal
from PySide6.QtGui import QColor, QDesktopServices
from PySide6.QtWidgets import QHBoxLayout, QLabel, QPushButton, QVBoxLayout
from qfluentwidgets.components.dialog_box import mask_dialog_base

from starrail.gui.common.stylesheet import StyleSheet
from starrail.utils import babelfish

AF = Qt.AlignmentFlag


class CheckUpdateDialog(mask_dialog_base.MaskDialogBase):

    cancelSignal = Signal()
    ndSignal = Signal()
    ghSignal = Signal()

    def __init__(
        self,
        title: str,
        content: str,
        parent: QtWidgets.QWidget = None,
        dist: Dict[str, str] = dict(),
    ):
        super().__init__(parent=parent)
        self.dist = copy.deepcopy(dist)

        self.__setupUi(title, content, self.widget)
        self.__initQss()
        self.__initLayout()
        self.__initWidget()

        self.setShadowEffect(60, (0, 10), QColor(0, 0, 0, 50))
        self.setMaskColor(QColor(0, 0, 0, 76))
        self._hBoxLayout.removeWidget(self.widget)
        self._hBoxLayout.addWidget(self.widget, 1, AF.AlignCenter)

        self.buttonGroup.setMinimumWidth(200)
        self.widget.setFixedSize(
            max(self.contentLabel.width(), self.titleLabel.width(), 150) + 48,
            self.contentLabel.y() + self.contentLabel.height() + 105,
        )
        self.__adjustText()

    def __setupUi(self, title, content, parent):
        self.title = title
        self.content = content
        self.titleLabel = QLabel(title, parent)
        self.contentLabel = QLabel(content, parent)

        self.buttonGroup = QtWidgets.QFrame(parent)
        if self.dist is not None:
            cancelText = babelfish.ui_cancel_update()
            ndText = babelfish.translate('netdisk_dist')
            ghText = babelfish.translate('GitHub_dist')
        else:
            cancelText = babelfish.ui_not_good()
            ndText = babelfish.ui_fine1()
            ghText = babelfish.ui_fine2()
        self.cancelButton = QPushButton(cancelText, self.buttonGroup)
        self.ndButton = qfw.PrimaryPushButton(ndText, self.buttonGroup)
        self.ghButton = qfw.PrimaryPushButton(ghText, self.buttonGroup)

        self.vBoxLayout = QVBoxLayout(parent)
        self.textLayout = QVBoxLayout()
        self.buttonLayout = QHBoxLayout(self.buttonGroup)

    def __initWidget(self):
        self.cancelButton.setAttribute(
            Qt.WidgetAttribute.WA_LayoutUsesWidgetRect,
        )
        self.ndButton.setAttribute(Qt.WidgetAttribute.WA_LayoutUsesWidgetRect)
        self.ghButton.setAttribute(Qt.WidgetAttribute.WA_LayoutUsesWidgetRect)

        self.cancelButton.setFocus()
        self.buttonGroup.setFixedHeight(81)
        self.__adjustText()

        self.cancelButton.clicked.connect(self.onCancelClicked)
        self.ndButton.clicked.connect(self.onNdClicked)
        self.ghButton.clicked.connect(self.onGhClicked)

    def __initLayout(self):
        self.vBoxLayout.setSpacing(0)
        self.vBoxLayout.setContentsMargins(0, 0, 0, 0)
        self.vBoxLayout.addLayout(self.textLayout, 1)
        self.vBoxLayout.addWidget(self.buttonGroup, 0, AF.AlignBottom)
        self.vBoxLayout.setSizeConstraint(QVBoxLayout.SetMinimumSize)

        self.textLayout.setSpacing(12)
        self.textLayout.setContentsMargins(24, 24, 24, 24)
        self.textLayout.addWidget(self.titleLabel, 0, AF.AlignTop)
        self.textLayout.addWidget(self.contentLabel, 0, AF.AlignTop)

        self.buttonLayout.setSpacing(12)
        self.buttonLayout.setContentsMargins(24, 24, 24, 24)
        self.buttonLayout.addWidget(self.cancelButton, 1, AF.AlignVCenter)
        self.buttonLayout.addWidget(self.ndButton, 1, AF.AlignVCenter)
        self.buttonLayout.addWidget(self.ghButton, 1, AF.AlignVCenter)

    def __initQss(self):
        self.titleLabel.setObjectName('titleLabel')
        self.contentLabel.setObjectName('contentLabel')
        self.buttonGroup.setObjectName('buttonGroup')
        self.cancelButton.setObjectName('cancelButton')

        StyleSheet.CHECK_UPDATE_DIALOG.apply(self)

        self.cancelButton.adjustSize()
        self.ghButton.adjustSize()
        self.ndButton.adjustSize()

    def __adjustText(self):
        # if self.isWindow():
        #     if self.parent():
        #         w = max(self.titleLabel.width(), self.parent().width())
        #         chars = max(min(w / 9, 140), 30)
        #     else:
        #         chars = 100
        # else:
        #     w = max(self.titleLabel.width(), self.window().width())
        #     chars = max(min(w / 9, 100), 30)

        # self.contentLabel.setText(
        #     qfw.TextWrap.wrap(self.content, chars, False)[0],
        # )
        pass

    def onCancelClicked(self):
        self.reject()

    def onGhClicked(self):
        self.accept()
        if self.dist is not None and self.dist['GitHub_dist']:
            QDesktopServices.openUrl(QUrl(self.dist['GitHub_dist']))

    def onNdClicked(self):
        self.accept()
        if self.dist is not None and self.dist['netdisk_dist']:
            QDesktopServices.openUrl(QUrl(self.dist['netdisk_dist']))

    def eventFilter(self, obj, e: QEvent):
        if obj is self.window():
            if e.type() == QEvent.Resize:
                self.__adjustText()

        return super().eventFilter(obj, e)



import traceback

import qfluentwidgets as qfw
from PySide6.QtCore import Qt

from starrail import __version__, digital_version
from starrail.gui.widgets.dialog import CheckUpdateDialog
from starrail.utils import babelfish
from starrail.utils.auto_update import check_update


def checkUpdate(parent, show_success=False, show_failure=True):
    try:
        latest = check_update()
        current = digital_version(__version__)
        if current < digital_version(latest.version):
            dialog = CheckUpdateDialog(
                title=babelfish.ui_update_available(),
                content=babelfish.ui_update_desc(latest.changelog),
                parent=parent,
                dist=latest.dist,
            )
            dialog.show()
            dialog.raise_()
        elif show_success:
            qfw.InfoBar.success(
                title=babelfish.ui_check_update_success(),
                content=babelfish.ui_is_latest_version(),
                orient=Qt.Orientation.Horizontal,
                isClosable=True,
                duration=3000,
                position=qfw.InfoBarPosition.TOP_RIGHT,
                parent=parent,
            )
    except Exception:
        traceback.print_exc()
        if show_failure:
            dialog = CheckUpdateDialog(
                title=babelfish.ui_ooops(),
                content=babelfish.ui_check_update_fail(),
                parent=parent,
                dist=None,
            )
            dialog.show()
            dialog.raise_()



import qfluentwidgets as qfw
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel, QWidget
from qfluentwidgets import FluentIcon

from starrail import __version__
from starrail.gui.common.config import qcfg
from starrail.gui.common.stylesheet import StyleSheet
from starrail.gui.common.utils import checkUpdate
from starrail.utils import babelfish


class SettingInterface(qfw.ScrollArea):

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.initialParent = parent
        self.scrollWidget = QWidget()
        self.expandLayout = qfw.ExpandLayout(self.scrollWidget)

        self.settingLabel = QLabel(babelfish.ui_settings(), self)

        self.personalGroup = qfw.SettingCardGroup(
            title=babelfish.ui_personalization(),
            parent=self.scrollWidget,
        )
        self.themeModeCard = qfw.OptionsSettingCard(
            configItem=qcfg.themeMode,
            icon=FluentIcon.BRUSH,
            title=babelfish.ui_theme_mode(),
            content=babelfish.ui_theme_mode_settings_desc(),
            texts=[
                babelfish.ui_theme_mode_light(),
                babelfish.ui_theme_mode_dark(),
                babelfish.ui_auto(),
            ],
            parent=self.personalGroup,
        )
        self.localeCard = qfw.OptionsSettingCard(
            configItem=qcfg.locale,
            icon=qfw.FluentIcon.LANGUAGE,
            title=babelfish.ui_locale(),
            content=babelfish.ui_locale_setting_desc(),
            texts=[
                babelfish.ui_auto(),
                babelfish.ui_zhs(),
                babelfish.ui_en(),
            ],
            parent=self.personalGroup,
        )
        self.checkUpdateCard = qfw.SwitchSettingCard(
            icon=qfw.FluentIcon.UPDATE,
            title=babelfish.ui_check_update_at_start(),
            content=babelfish.ui_check_update_at_start_desc(),
            configItem=qcfg.check_update,
            parent=self.personalGroup,
        )

        self.aboutGroup = qfw.SettingCardGroup(
            babelfish.ui_about(), self.scrollWidget,
        )
        self.getStartCard = qfw.HyperlinkCard(
            babelfish.constants.DOCS_URL,
            babelfish.ui_open_docs(),
            FluentIcon.BOOK_SHELF,
            babelfish.ui_get_started(),
            babelfish.ui_get_started_desc(),
            self.aboutGroup,
        )
        self.troubleshootingCard = qfw.HyperlinkCard(
            babelfish.constants.TROUBLESHOOTING_URL,
            babelfish.ui_open_troubleshooting(),
            FluentIcon.HELP,
            babelfish.ui_troubleshooting(),
            babelfish.ui_troubleshooting_desc(),
            self.aboutGroup,
        )
        self.openSourceCard = qfw.HyperlinkCard(
            babelfish.constants.REPO_URL,
            babelfish.ui_open_repo(),
            FluentIcon.GITHUB,
            babelfish.ui_github_repo(),
            babelfish.ui_github_repo_desc(),
            self.aboutGroup,
        )
        self.feedbackCard = qfw.HyperlinkCard(
            babelfish.constants.ISSUE_URL,
            babelfish.ui_open_issues(),
            FluentIcon.FEEDBACK,
            babelfish.ui_send_feedback(),
            babelfish.ui_send_feedback_desc(),
            self.aboutGroup,
        )
        self.aboutCard = qfw.PrimaryPushSettingCard(
            babelfish.ui_check_update(),
            FluentIcon.INFO,
            babelfish.ui_about_this(),
            (
                f'{babelfish.ui_copyright(2023)} '
                f'{babelfish.ui_current_version(__version__)}'
            ),
            self.aboutGroup,
        )
        self.aboutCard.button.clicked.connect(self.checkUpdateAction)

        self.__initWidget()
        self.__initLayout()
        self.__connectSignalToSlot()

    def __initWidget(self):
        self.resize(1000, 800)
        SBP = Qt.ScrollBarPolicy
        self.setHorizontalScrollBarPolicy(SBP.ScrollBarAlwaysOff)
        self.setViewportMargins(0, 80, 0, 20)
        self.setWidget(self.scrollWidget)
        self.setWidgetResizable(True)

        self.scrollWidget.setObjectName('scrollWidget')
        self.settingLabel.setObjectName('settingLabel')
        StyleSheet.SETTING_INTERFACE.apply(self)

    def __initLayout(self):
        self.settingLabel.move(36, 30)

        self.personalGroup.addSettingCard(self.themeModeCard)
        self.personalGroup.addSettingCard(self.localeCard)
        self.personalGroup.addSettingCard(self.checkUpdateCard)

        self.aboutGroup.addSettingCard(self.getStartCard)
        self.aboutGroup.addSettingCard(self.troubleshootingCard)
        self.aboutGroup.addSettingCard(self.openSourceCard)
        self.aboutGroup.addSettingCard(self.feedbackCard)
        self.aboutGroup.addSettingCard(self.aboutCard)

        self.expandLayout.setSpacing(28)
        self.expandLayout.setContentsMargins(36, 10, 36, 0)
        self.expandLayout.addWidget(self.personalGroup)
        self.expandLayout.addWidget(self.aboutGroup)

    def __showRestartToolTip(self):
        qfw.InfoBar.success(
            title=babelfish.ui_setting_restart_title(),
            content=babelfish.ui_setting_restart_content(),
            orient=Qt.Orientation.Horizontal,
            isClosable=True,
            duration=2000,
            position=qfw.InfoBarPosition.TOP_RIGHT,
            parent=self,
        )

    def __connectSignalToSlot(self):
        qcfg.themeChanged.connect(qfw.setTheme)
        qcfg.appRestartSig.connect(self.__showRestartToolTip)

    def checkUpdateAction(self):
        checkUpdate(parent=self.initialParent, show_success=True)



from PySide6.QtCore import Qt
from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QHBoxLayout, QLabel, QVBoxLayout
from qframelesswindow import TitleBar


class CustomTitleBar(TitleBar):
    """ Title bar with icon and title """

    def __init__(self, parent):
        super().__init__(parent)
        self.setFixedHeight(48)
        self.hBoxLayout.removeWidget(self.minBtn)
        self.hBoxLayout.removeWidget(self.maxBtn)
        self.hBoxLayout.removeWidget(self.closeBtn)

        # add window icon
        self.iconLabel = QLabel(self)
        self.iconLabel.setFixedSize(18, 18)
        self.hBoxLayout.insertSpacing(0, 10)
        self.hBoxLayout.insertWidget(
            1, self.iconLabel, 0, Qt.AlignLeft | Qt.AlignVCenter,
        )
        self.window().windowIconChanged.connect(self.setIcon)

        # add title label
        self.titleLabel = QLabel(self)
        self.hBoxLayout.insertWidget(
            2, self.titleLabel, 0, Qt.AlignLeft | Qt.AlignVCenter,
        )
        self.titleLabel.setObjectName('titleLabel')
        self.window().windowTitleChanged.connect(self.setTitle)

        self.vBoxLayout = QVBoxLayout()
        self.buttonLayout = QHBoxLayout()
        self.buttonLayout.setSpacing(0)
        self.buttonLayout.setContentsMargins(0, 0, 0, 0)
        self.buttonLayout.setAlignment(Qt.AlignTop)
        self.buttonLayout.addWidget(self.minBtn)
        self.buttonLayout.addWidget(self.maxBtn)
        self.buttonLayout.addWidget(self.closeBtn)
        self.vBoxLayout.addLayout(self.buttonLayout)
        self.vBoxLayout.addStretch(1)
        self.hBoxLayout.addLayout(self.vBoxLayout, 0)

    def setTitle(self, title):
        self.titleLabel.setText(title)
        self.titleLabel.adjustSize()

    def setIcon(self, icon):
        self.iconLabel.setPixmap(QIcon(icon).pixmap(18, 18))
